@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapsearch.c,v 1.170.2.2 2003/09/18 15:43:28 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapsearch.c,v 1.170.2.4 2003/10/13 01:57:01 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -995,7 +995,7 @@ done:
 	return( rc );
 }
 
-#if 1
+#ifdef OLD_WAY
 /* This is the original version, the old way of doing things. */
 static void
 print_entry(
@@ -1131,11 +1131,10 @@ print_entry(
 	}
 	write_ldif( LDIF_PUT_VALUE, "dn", bv.bv_val, bv.bv_len );
 
-	rc = ldap_int_get_controls( ber, &ctrls );
-
+	rc = ldap_pvt_get_controls( ber, &ctrls );
 	if( rc != LDAP_SUCCESS ) {
 		fprintf(stderr, _("print_entry: %d\n"), rc );
-		ldap_perror( ld, "ldap_get_entry_controls" );
+		ldap_perror( ld, "ldap_pvt_get_controls" );
 		exit( EXIT_FAILURE );
 	}
 
@@ -1161,10 +1160,10 @@ print_entry(
 	{
 		if (bv.bv_val == NULL) break;
 
-		if ( attrsonly || !bvals ) {
+		if ( attrsonly ) {
 			write_ldif( LDIF_PUT_NOVALUE, bv.bv_val, NULL, 0 );
 
-		} else {
+		} else if ( bvals ) {
 			for ( i = 0; bvals[i].bv_val != NULL; i++ ) {
 				if ( vals2tmp > 1 || ( vals2tmp
 					&& ldif_is_not_printable( bvals[i].bv_val, bvals[i].bv_len ) ))
