@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/include/ldap.h,v 1.168.2.3 2002/04/18 15:39:20 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/include/ldap.h,v 1.208.2.3 2003/09/18 15:43:31 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, Redwood City, California, USA
+ * Copyright 1998-2003 The OpenLDAP Foundation, Redwood City, California, USA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -95,7 +95,8 @@ LDAP_BEGIN_DECL
 
 /* 0x16 - 0x2f not defined by current draft */
 #define LDAP_OPT_HOST_NAME			0x0030
-#define	LDAP_OPT_ERROR_NUMBER		0x0031
+#define LDAP_OPT_RESULT_CODE		0x0031
+#define LDAP_OPT_ERROR_NUMBER		LDAP_OPT_RESULT_CODE
 #define LDAP_OPT_ERROR_STRING		0x0032
 #define LDAP_OPT_MATCHED_DN			0x0033
 
@@ -108,8 +109,9 @@ LDAP_BEGIN_DECL
 #define LDAP_OPT_DEBUG_LEVEL		0x5001	/* debug level */
 #define LDAP_OPT_TIMEOUT			0x5002	/* default timeout */
 #define LDAP_OPT_REFHOPLIMIT		0x5003	/* ref hop limit */
-#define LDAP_OPT_NETWORK_TIMEOUT        0x5005  /* socket level timeout */
+#define LDAP_OPT_NETWORK_TIMEOUT	0x5005	/* socket level timeout */
 #define LDAP_OPT_URI				0x5006
+#define LDAP_OPT_REFERRAL_URLS      0x5007  /* Referral URLs */
 
 /* OpenLDAP TLS options */
 #define LDAP_OPT_X_TLS				0x6000
@@ -142,9 +144,8 @@ LDAP_BEGIN_DECL
 #define LDAP_OPT_X_SASL_SSF_MAX			0x6108
 #define	LDAP_OPT_X_SASL_MAXBUFSIZE		0x6109
 
-
 /* on/off values */
-#define LDAP_OPT_ON		((void *) 1)
+#define LDAP_OPT_ON		((void *) &ber_pvt_opt_on)
 #define LDAP_OPT_OFF	((void *) 0)
 
 /*
@@ -180,14 +181,54 @@ typedef struct ldapcontrol {
 } LDAPControl;
 
 /* LDAP Controls */
+#define LDAP_CONTROL_ASSERT				"1.3.6.1.4.1.4203.666.5.9"
+#define LDAP_CONTROL_PRE_READ			"1.3.6.1.4.1.4203.666.5.10.1"
+#define LDAP_CONTROL_POST_READ			"1.3.6.1.4.1.4203.666.5.10.2"
+#define LDAP_CONTROL_MODIFY_INCREMENT	"1.3.6.1.4.1.4203.666.5.11"
 
-#define LDAP_CONTROL_MANAGEDSAIT	"2.16.840.1.113730.3.4.2"
+#define LDAP_CONTROL_VALUESRETURNFILTER	"1.2.826.0.1.334810.2.3"
+#define LDAP_CONTROL_SUBENTRIES			"1.3.6.1.4.1.4203.1.10.1"
+#define LDAP_CONTROL_NOOP				"1.3.6.1.4.1.4203.1.10.2"
+#define LDAP_CONTROL_MANAGEDSAIT		"2.16.840.1.113730.3.4.2"
+#define LDAP_CONTROL_PROXY_AUTHZ		"2.16.840.1.113730.3.4.18"
+
+#if 0
+#define LDAP_CONTROL_DUPENT_REQUEST		"2.16.840.1.113719.1.27.101.1"
+#define LDAP_CONTROL_DUPENT_RESPONSE	"2.16.840.1.113719.1.27.101.2"
+#define LDAP_CONTROL_DUPENT_ENTRY		"2.16.840.1.113719.1.27.101.3"
+#define LDAP_CONTROL_DUPENT	LDAP_CONTROL_DUPENT_REQUEST
+#endif
+
+#define LDAP_CONTROL_PAGEDRESULTS		"1.2.840.113556.1.4.319"
+
+#define LDAP_CONTROL_SYNC		"1.3.6.1.4.1.4203.666.5.6"
+#define LDAP_CONTROL_SYNC_STATE	"1.3.6.1.4.1.4203.666.5.7"
+#define LDAP_CONTROL_SYNC_DONE	"1.3.6.1.4.1.4203.666.5.8"
+#define LDAP_SYNC_INFO			"1.3.6.1.4.1.4203.666.10.2"
+
+#define LDAP_SYNC_NEW_COOKIE		0
+#define LDAP_SYNC_STATE_MODE_DONE	1
+#define	LDAP_SYNC_LOG_MODE_DONE		2
+#define LDAP_SYNC_REFRESH_DONE		3
+
+#define LDAP_SYNC_STATE_MODE		0
+#define LDAP_SYNC_LOG_MODE			1
+#define LDAP_SYNC_PERSIST_MODE		2
+
+#define LDAP_SYNC_PRESENT		0
+#define LDAP_SYNC_ADD			1
+#define LDAP_SYNC_MODIFY		2
+#define LDAP_SYNC_DELETE		3
 
 #define LDAP_CONTROL_SORTREQUEST    "1.2.840.113556.1.4.473"
 #define LDAP_CONTROL_SORTRESPONSE	"1.2.840.113556.1.4.474"
 #define LDAP_CONTROL_VLVREQUEST    	"2.16.840.1.113730.3.4.9"
 #define LDAP_CONTROL_VLVRESPONSE    "2.16.840.1.113730.3.4.10"
 
+/* controls for MSAD compatibility */
+#define LDAP_CONTROL_X_DOMAIN_SCOPE "1.2.840.113556.1.4.1339"
+#define LDAP_CONTROL_X_PERMISSIVE_MODIFY "1.2.840.113556.1.4.1413"
+
 /* LDAP Unsolicited Notifications */
 #define	LDAP_NOTICE_OF_DISCONNECTION	"1.3.6.1.4.1.1466.20036"
 #define LDAP_NOTICE_DISCONNECT LDAP_NOTICE_OF_DISCONNECTION
@@ -202,6 +243,15 @@ typedef struct ldapcontrol {
 #define LDAP_TAG_EXOP_MODIFY_PASSWD_GEN	((ber_tag_t) 0x80U)
 
 #define LDAP_EXOP_X_WHO_AM_I	"1.3.6.1.4.1.4203.1.11.3"
+#define LDAP_EXOP_X_CANCEL		"1.3.6.1.4.1.4203.666.6.3"
+
+/* LDAP Features */
+#define LDAP_FEATURE_ALL_OPERATIONAL_ATTRS "1.3.6.1.4.1.4203.1.5.1"  /* + */
+#define LDAP_FEATURE_OBJECTCLASS_ATTRS "1.3.6.1.4.1.4203.1.5.2"
+#define LDAP_FEATURE_ABSOLUTE_FILTERS "1.3.6.1.4.1.4203.1.5.3"  /* (&) (|) */
+#define LDAP_FEATURE_LANGUAGE_TAG_OPTIONS "1.3.6.1.4.1.4203.1.5.4"
+#define LDAP_FEATURE_LANGUAGE_RANGE_OPTIONS "1.3.6.1.4.1.4203.1.5.5"
+#define LDAP_FEATURE_MODIFY_INCREMENT "1.3.6.1.4.1.4203.666.5.6"
 
 /*
  * specific LDAP instantiations of BER types we know about
@@ -227,8 +277,10 @@ typedef struct ldapcontrol {
 /* general stuff */
 #define LDAP_TAG_MESSAGE	((ber_tag_t) 0x30U)	/* constructed + 16 */
 #define LDAP_TAG_MSGID		((ber_tag_t) 0x02U)	/* integer */
+
 #define LDAP_TAG_LDAPDN		((ber_tag_t) 0x04U)	/* octet string */
 #define LDAP_TAG_LDAPCRED	((ber_tag_t) 0x04U)	/* octet string */
+
 #define LDAP_TAG_CONTROLS	((ber_tag_t) 0xa0U)	/* context specific + constructed + 0 */
 #define LDAP_TAG_REFERRAL	((ber_tag_t) 0xa3U)	/* context specific + constructed + 3 */
 
@@ -241,75 +293,77 @@ typedef struct ldapcontrol {
 
 #define LDAP_TAG_SASL_RES_CREDS	((ber_tag_t) 0x87U)	/* context specific + primitive */
 
+#define LDAP_SYNC_TAG_COOKIE	((ber_tag_t) 0x04U)	/* octet string */
+
 
 /* possible operations a client can invoke */
-#define LDAP_REQ_BIND			((ber_tag_t) 0x60U)	/* application + constructed */
-#define LDAP_REQ_UNBIND			((ber_tag_t) 0x42U)	/* application + primitive   */
-#define LDAP_REQ_SEARCH			((ber_tag_t) 0x63U)	/* application + constructed */
-#define LDAP_REQ_MODIFY			((ber_tag_t) 0x66U)	/* application + constructed */
-#define LDAP_REQ_ADD			((ber_tag_t) 0x68U)	/* application + constructed */
-#define LDAP_REQ_DELETE			((ber_tag_t) 0x4aU)	/* application + primitive   */
-#define LDAP_REQ_MODDN			((ber_tag_t) 0x6cU)	/* application + constructed */
-#define LDAP_REQ_MODRDN			LDAP_REQ_MODDN
-#define LDAP_REQ_RENAME			LDAP_REQ_MODDN
-#define LDAP_REQ_COMPARE		((ber_tag_t) 0x6eU)	/* application + constructed */
-#define LDAP_REQ_ABANDON		((ber_tag_t) 0x50U)	/* application + primitive   */
-#define LDAP_REQ_EXTENDED		((ber_tag_t) 0x77U)	/* application + constructed */
+#define LDAP_REQ_BIND		((ber_tag_t) 0x60U)	/* application + constructed */
+#define LDAP_REQ_UNBIND		((ber_tag_t) 0x42U)	/* application + primitive   */
+#define LDAP_REQ_SEARCH		((ber_tag_t) 0x63U)	/* application + constructed */
+#define LDAP_REQ_MODIFY		((ber_tag_t) 0x66U)	/* application + constructed */
+#define LDAP_REQ_ADD		((ber_tag_t) 0x68U)	/* application + constructed */
+#define LDAP_REQ_DELETE		((ber_tag_t) 0x4aU)	/* application + primitive   */
+#define LDAP_REQ_MODDN		((ber_tag_t) 0x6cU)	/* application + constructed */
+#define LDAP_REQ_MODRDN		LDAP_REQ_MODDN
+#define LDAP_REQ_RENAME		LDAP_REQ_MODDN
+#define LDAP_REQ_COMPARE	((ber_tag_t) 0x6eU)	/* application + constructed */
+#define LDAP_REQ_ABANDON	((ber_tag_t) 0x50U)	/* application + primitive   */
+#define LDAP_REQ_EXTENDED	((ber_tag_t) 0x77U)	/* application + constructed */
 
 /* possible result types a server can return */
-#define LDAP_RES_BIND			((ber_tag_t) 0x61U)	/* application + constructed */
-#define LDAP_RES_SEARCH_ENTRY		((ber_tag_t) 0x64U)	/* application + constructed */
+#define LDAP_RES_BIND		((ber_tag_t) 0x61U)	/* application + constructed */
+#define LDAP_RES_SEARCH_ENTRY	((ber_tag_t) 0x64U)	/* application + constructed */
 #define LDAP_RES_SEARCH_REFERENCE	((ber_tag_t) 0x73U)	/* V3: application + constructed */
-#define LDAP_RES_SEARCH_RESULT		((ber_tag_t) 0x65U)	/* application + constructed */
-#define LDAP_RES_MODIFY			((ber_tag_t) 0x67U)	/* application + constructed */
-#define LDAP_RES_ADD			((ber_tag_t) 0x69U)	/* application + constructed */
-#define LDAP_RES_DELETE			((ber_tag_t) 0x6bU)	/* application + constructed */
-#define LDAP_RES_MODDN			((ber_tag_t) 0x6dU)	/* application + constructed */
-#define LDAP_RES_MODRDN			LDAP_RES_MODDN	/* application + constructed */
-#define LDAP_RES_RENAME			LDAP_RES_MODDN	/* application + constructed */
-#define LDAP_RES_COMPARE		((ber_tag_t) 0x6fU)	/* application + constructed */
-#define LDAP_RES_EXTENDED		((ber_tag_t) 0x78U)	/* V3: application + constructed */
-#define LDAP_RES_EXTENDED_PARTIAL	((ber_tag_t) 0x79U)	/* V3+: application + constructed */
+#define LDAP_RES_SEARCH_RESULT	((ber_tag_t) 0x65U)	/* application + constructed */
+#define LDAP_RES_MODIFY		((ber_tag_t) 0x67U)	/* application + constructed */
+#define LDAP_RES_ADD		((ber_tag_t) 0x69U)	/* application + constructed */
+#define LDAP_RES_DELETE		((ber_tag_t) 0x6bU)	/* application + constructed */
+#define LDAP_RES_MODDN		((ber_tag_t) 0x6dU)	/* application + constructed */
+#define LDAP_RES_MODRDN		LDAP_RES_MODDN	/* application + constructed */
+#define LDAP_RES_RENAME		LDAP_RES_MODDN	/* application + constructed */
+#define LDAP_RES_COMPARE	((ber_tag_t) 0x6fU)	/* application + constructed */
+#define LDAP_RES_EXTENDED	((ber_tag_t) 0x78U)	/* V3: application + constructed */
+#define LDAP_RES_INTERMEDIATE	((ber_tag_t) 0x79U) /* V3+: application + constructed */
 
 #define LDAP_RES_ANY			(-1)
 #define LDAP_RES_UNSOLICITED	(0)
 
 
 /* sasl methods */
-#define LDAP_SASL_SIMPLE		((char*)0)
-#define LDAP_SASL_NULL			("")
+#define LDAP_SASL_SIMPLE	((char*)0)
+#define LDAP_SASL_NULL		("")
 
 
 /* authentication methods available */
-#define LDAP_AUTH_NONE		((ber_tag_t) 0x00U)	/* no authentication		  */
-#define LDAP_AUTH_SIMPLE	((ber_tag_t) 0x80U)	/* context specific + primitive   */
-#define LDAP_AUTH_SASL		((ber_tag_t) 0xa3U)	/* context specific + constructed */
-#define LDAP_AUTH_KRBV4		((ber_tag_t) 0xffU)	/* means do both of the following */
-#define LDAP_AUTH_KRBV41	((ber_tag_t) 0x81U)	/* context specific + primitive   */
-#define LDAP_AUTH_KRBV42	((ber_tag_t) 0x82U)	/* context specific + primitive   */
+#define LDAP_AUTH_NONE	((ber_tag_t) 0x00U)	/* no authentication		  */
+#define LDAP_AUTH_SIMPLE	((ber_tag_t) 0x80U)	/* context specific + primitive */
+#define LDAP_AUTH_SASL	((ber_tag_t) 0xa3U)	/* context specific + constructed */
+#define LDAP_AUTH_KRBV4	((ber_tag_t) 0xffU)	/* means do both of the following */
+#define LDAP_AUTH_KRBV41	((ber_tag_t) 0x81U)	/* context specific + primitive */
+#define LDAP_AUTH_KRBV42	((ber_tag_t) 0x82U)	/* context specific + primitive */
 
 
 /* filter types */
-#define LDAP_FILTER_AND		((ber_tag_t) 0xa0U)	/* context specific + constructed */
-#define LDAP_FILTER_OR		((ber_tag_t) 0xa1U)	/* context specific + constructed */
-#define LDAP_FILTER_NOT		((ber_tag_t) 0xa2U)	/* context specific + constructed */
+#define LDAP_FILTER_AND	((ber_tag_t) 0xa0U)	/* context specific + constructed */
+#define LDAP_FILTER_OR	((ber_tag_t) 0xa1U)	/* context specific + constructed */
+#define LDAP_FILTER_NOT	((ber_tag_t) 0xa2U)	/* context specific + constructed */
 #define LDAP_FILTER_EQUALITY	((ber_tag_t) 0xa3U)	/* context specific + constructed */
 #define LDAP_FILTER_SUBSTRINGS	((ber_tag_t) 0xa4U)	/* context specific + constructed */
-#define LDAP_FILTER_GE		((ber_tag_t) 0xa5U)	/* context specific + constructed */
-#define LDAP_FILTER_LE		((ber_tag_t) 0xa6U)	/* context specific + constructed */
+#define LDAP_FILTER_GE	((ber_tag_t) 0xa5U)	/* context specific + constructed */
+#define LDAP_FILTER_LE	((ber_tag_t) 0xa6U)	/* context specific + constructed */
 #define LDAP_FILTER_PRESENT	((ber_tag_t) 0x87U)	/* context specific + primitive   */
 #define LDAP_FILTER_APPROX	((ber_tag_t) 0xa8U)	/* context specific + constructed */
-#define LDAP_FILTER_EXT		((ber_tag_t) 0xa9U)	/* context specific + constructed */
+#define LDAP_FILTER_EXT	((ber_tag_t) 0xa9U)	/* context specific + constructed */
 
 /* extended filter component types */
-#define LDAP_FILTER_EXT_OID	((ber_tag_t) 0x81U)	/* context specific */
+#define LDAP_FILTER_EXT_OID		((ber_tag_t) 0x81U)	/* context specific */
 #define LDAP_FILTER_EXT_TYPE	((ber_tag_t) 0x82U)	/* context specific */
 #define LDAP_FILTER_EXT_VALUE	((ber_tag_t) 0x83U)	/* context specific */
 #define LDAP_FILTER_EXT_DNATTRS	((ber_tag_t) 0x84U)	/* context specific */
 
 /* substring filter component types */
 #define LDAP_SUBSTRING_INITIAL	((ber_tag_t) 0x80U)	/* context specific */
-#define LDAP_SUBSTRING_ANY	((ber_tag_t) 0x81U)	/* context specific */
+#define LDAP_SUBSTRING_ANY		((ber_tag_t) 0x81U)	/* context specific */
 #define LDAP_SUBSTRING_FINAL	((ber_tag_t) 0x82U)	/* context specific */
 
 /* search scopes */
@@ -320,7 +374,7 @@ typedef struct ldapcontrol {
 
 /* substring filter component types */
 #define LDAP_SUBSTRING_INITIAL	((ber_tag_t) 0x80U)	/* context specific */
-#define LDAP_SUBSTRING_ANY	((ber_tag_t) 0x81U)	/* context specific */
+#define LDAP_SUBSTRING_ANY		((ber_tag_t) 0x81U)	/* context specific */
 #define LDAP_SUBSTRING_FINAL	((ber_tag_t) 0x82U)	/* context specific */
 
 /*
@@ -329,13 +383,13 @@ typedef struct ldapcontrol {
 
 #define LDAP_RANGE(n,x,y)	(((x) <= (n)) && ((n) <= (y)))
 
-#define LDAP_SUCCESS			0x00
+#define LDAP_SUCCESS				0x00
 #define LDAP_OPERATIONS_ERROR		0x01
-#define LDAP_PROTOCOL_ERROR		0x02
+#define LDAP_PROTOCOL_ERROR			0x02
 #define LDAP_TIMELIMIT_EXCEEDED		0x03
 #define LDAP_SIZELIMIT_EXCEEDED		0x04
-#define LDAP_COMPARE_FALSE		0x05
-#define LDAP_COMPARE_TRUE		0x06
+#define LDAP_COMPARE_FALSE			0x05
+#define LDAP_COMPARE_TRUE			0x06
 #define LDAP_AUTH_METHOD_NOT_SUPPORTED	0x07
 #define LDAP_STRONG_AUTH_NOT_SUPPORTED	LDAP_AUTH_METHOD_NOT_SUPPORTED
 #define LDAP_STRONG_AUTH_REQUIRED	0x08
@@ -350,32 +404,33 @@ typedef struct ldapcontrol {
 #define LDAP_ATTR_ERROR(n)	LDAP_RANGE((n),0x10,0x15) /* 16-21 */
 
 #define LDAP_NO_SUCH_ATTRIBUTE		0x10
-#define LDAP_UNDEFINED_TYPE		0x11
+#define LDAP_UNDEFINED_TYPE			0x11
 #define LDAP_INAPPROPRIATE_MATCHING	0x12
 #define LDAP_CONSTRAINT_VIOLATION	0x13
 #define LDAP_TYPE_OR_VALUE_EXISTS	0x14
-#define LDAP_INVALID_SYNTAX		0x15
+#define LDAP_INVALID_SYNTAX			0x15
 
 #define LDAP_NAME_ERROR(n)	LDAP_RANGE((n),0x20,0x24) /* 32-34,36 */
 
-#define LDAP_NO_SUCH_OBJECT		0x20
-#define LDAP_ALIAS_PROBLEM		0x21
+#define LDAP_NO_SUCH_OBJECT			0x20
+#define LDAP_ALIAS_PROBLEM			0x21
 #define LDAP_INVALID_DN_SYNTAX		0x22
-#define LDAP_IS_LEAF			0x23 /* not LDAPv3 */
+#define LDAP_IS_LEAF				0x23 /* not LDAPv3 */
 #define LDAP_ALIAS_DEREF_PROBLEM	0x24
 
-#define LDAP_SECURITY_ERROR(n)	LDAP_RANGE((n),0x30,0x32) /* 48-50 */
+#define LDAP_SECURITY_ERROR(n)	LDAP_RANGE((n),0x2F,0x32) /* 47-50 */
 
+#define LDAP_PROXY_AUTHZ_FAILURE	0x2F /* LDAPv3 proxy authorization */
 #define LDAP_INAPPROPRIATE_AUTH		0x30
 #define LDAP_INVALID_CREDENTIALS	0x31
 #define LDAP_INSUFFICIENT_ACCESS	0x32
 
 #define LDAP_SERVICE_ERROR(n)	LDAP_RANGE((n),0x33,0x36) /* 51-54 */
 
-#define LDAP_BUSY			0x33
-#define LDAP_UNAVAILABLE		0x34
+#define LDAP_BUSY					0x33
+#define LDAP_UNAVAILABLE			0x34
 #define LDAP_UNWILLING_TO_PERFORM	0x35
-#define LDAP_LOOP_DETECT		0x36
+#define LDAP_LOOP_DETECT			0x36
 
 #define LDAP_UPDATE_ERROR(n)	LDAP_RANGE((n),0x40,0x47) /* 64-69,71 */
 
@@ -383,16 +438,16 @@ typedef struct ldapcontrol {
 #define LDAP_OBJECT_CLASS_VIOLATION	0x41
 #define LDAP_NOT_ALLOWED_ON_NONLEAF	0x42
 #define LDAP_NOT_ALLOWED_ON_RDN		0x43
-#define LDAP_ALREADY_EXISTS		0x44
+#define LDAP_ALREADY_EXISTS			0x44
 #define LDAP_NO_OBJECT_CLASS_MODS	0x45
 #define LDAP_RESULTS_TOO_LARGE		0x46 /* CLDAP */
 #define LDAP_AFFECTS_MULTIPLE_DSAS	0x47 /* LDAPv3 */
 
-#define LDAP_OTHER			0x50
+#define LDAP_OTHER				0x50
 
 #define LDAP_API_ERROR(n)		LDAP_RANGE((n),0x51,0x61) /* 81-97 */
 #define LDAP_API_RESULT(n)		(((n) == LDAP_SUCCESS) || \
-									LDAP_RANGE((n),0x51,0x61)) /* 0,81-97 */
+								LDAP_RANGE((n),0x51,0x61)) /* 0,81-97 */
 
 /* reserved for APIs */
 #define LDAP_SERVER_DOWN		0x51
@@ -415,6 +470,28 @@ typedef struct ldapcontrol {
 #define LDAP_CLIENT_LOOP				0x60	/* draft-ietf-ldap-c-api-xx */
 #define LDAP_REFERRAL_LIMIT_EXCEEDED	0x61	/* draft-ietf-ldap-c-api-xx */
 
+#define LDAP_SYNC_RESOURCES_EXHAUSTED	0x100
+#define LDAP_SYNC_SECURITY_VIOLATION	0x101
+#define LDAP_SYNC_INVALID_COOKIE		0x102
+#define LDAP_SYNC_UNSUPPORTED_SCHEME	0x103
+#define LDAP_SYNC_CLIENT_DISCONNECT		0x104
+#define LDAP_SYNC_RELOAD_REQUIRED		0x105
+
+#define LDAP_ASSERTION_FAILED			0x10f
+
+#ifdef LDAP_EXOP_X_CANCEL
+/* resultCode for Cancel Response */
+#define LDAP_CANCELLED					0x110
+#define LDAP_NO_SUCH_OPERATION			0x111
+#define LDAP_TOO_LATE					0x112
+#define LDAP_CANNOT_CANCEL				0x113
+#endif
+
+/* LDAP SYNC request type */
+#define LDAP_SYNC_NONE					0x00
+#define LDAP_SYNC_REFRESH_ONLY			0x01
+#define LDAP_SYNC_REFRESH_AND_PERSIST	0x03
+
 /*
  * This structure represents both ldap messages and ldap responses.
  * These are really the same, except in the case of search responses,
@@ -427,10 +504,12 @@ typedef struct ldapmsg LDAPMessage;
 typedef struct ldapmod {
 	int		mod_op;
 
-#define LDAP_MOD_ADD		((ber_int_t) 0x0000)
-#define LDAP_MOD_DELETE		((ber_int_t) 0x0001)
-#define LDAP_MOD_REPLACE	((ber_int_t) 0x0002)
-#define LDAP_MOD_BVALUES	((ber_int_t) 0x0080)
+#define LDAP_MOD_OP			(0x0007)
+#define LDAP_MOD_ADD		(0x0000)
+#define LDAP_MOD_DELETE		(0x0001)
+#define LDAP_MOD_REPLACE	(0x0002)
+#define LDAP_MOD_INCREMENT	(0x0003)
+#define LDAP_MOD_BVALUES	(0x0080)
 /* IMPORTANT: do not use code 0x1000 (or above),
  * it is used internally by the backends!
  * (see ldap/servers/slapd/slap.h)
@@ -445,42 +524,6 @@ typedef struct ldapmod {
 #define mod_bvalues	mod_vals.modv_bvals
 } LDAPMod;
 
-/*
- * structures for ldap getfilter routines
- */
-
-typedef struct ldap_filt_info {
-	char			*lfi_filter;
-	char			*lfi_desc;
-	int			lfi_scope;
-	int			lfi_isexact;
-	struct ldap_filt_info	*lfi_next;
-} LDAPFiltInfo;
-
-typedef struct ldap_filt_list {
-    char			*lfl_tag;
-    char			*lfl_pattern;
-    char			*lfl_delims;
-    LDAPFiltInfo		*lfl_ilist;
-    struct ldap_filt_list	*lfl_next;
-} LDAPFiltList;
-
-
-#define LDAP_FILT_MAXSIZ	1024
-
-typedef struct ldap_filt_desc {
-	LDAPFiltList		*lfd_filtlist;
-	LDAPFiltInfo		*lfd_curfip;
-	LDAPFiltInfo		lfd_retfi;
-	char			lfd_filter[ LDAP_FILT_MAXSIZ ];
-	char			*lfd_curval;
-	char			*lfd_curvalcopy;
-	char			**lfd_curvalwords;
-	char			*lfd_filtprefix;
-	char			*lfd_filtsuffix;
-} LDAPFiltDesc;
-
-
 /*
  * structure representing an ldap session which can
  * encompass connections to multiple servers (in the
@@ -488,17 +531,17 @@ typedef struct ldap_filt_desc {
  */
 typedef struct ldap LDAP;
 
-#define LDAP_DEREF_NEVER	0x00
+#define LDAP_DEREF_NEVER		0x00
 #define LDAP_DEREF_SEARCHING	0x01
-#define LDAP_DEREF_FINDING	0x02
-#define LDAP_DEREF_ALWAYS	0x03
+#define LDAP_DEREF_FINDING		0x02
+#define LDAP_DEREF_ALWAYS		0x03
 
-#define LDAP_NO_LIMIT		0
+#define LDAP_NO_LIMIT			0
 
 /* how many messages to retrieve results for */
-#define LDAP_MSG_ONE		0x00
-#define LDAP_MSG_ALL		0x01
-#define LDAP_MSG_RECEIVED	0x02
+#define LDAP_MSG_ONE			0x00
+#define LDAP_MSG_ALL			0x01
+#define LDAP_MSG_RECEIVED		0x02
 
 /*
  * types for ldap URL handling
@@ -629,7 +672,7 @@ ldap_parse_extended_result LDAP_P((
 	int				freeit ));
 
 LDAP_F( int )
-ldap_parse_extended_partial LDAP_P((
+ldap_parse_intermediate LDAP_P((
 	LDAP			*ld,
 	LDAPMessage		*res,
 	char			**retoidp,
@@ -809,31 +852,24 @@ ldap_kerberos_bind2_s LDAP_P((	/* deprecated */
 	LDAP *ld,
 	LDAP_CONST char *who ));
 
+
 /*
- * in cache.c
- * (deprecated)
+ * LDAP Cancel Extended Operation <draft-zeilenga-ldap-cancel-xx.txt>
  */
+
 LDAP_F( int )
-ldap_enable_cache LDAP_P(( LDAP *ld, long timeout, ber_len_t maxmem ));
-
-LDAP_F( void )
-ldap_disable_cache LDAP_P(( LDAP *ld ));
-
-LDAP_F( void )
-ldap_set_cache_options LDAP_P(( LDAP *ld, unsigned long opts ));
-
-LDAP_F( void )
-ldap_destroy_cache LDAP_P(( LDAP *ld ));
-
-LDAP_F( void )
-ldap_flush_cache LDAP_P(( LDAP *ld ));
-
-LDAP_F( void )
-ldap_uncache_entry LDAP_P(( LDAP *ld, LDAP_CONST char *dn ));
-
-LDAP_F( void )
-ldap_uncache_request LDAP_P(( LDAP *ld, int msgid ));
+ldap_cancel LDAP_P(( LDAP *ld,
+	int cancelid,
+	LDAPControl		**sctrls,
+	LDAPControl		**cctrls,
+	int				*msgidp ));
 
+LDAP_F( int )
+ldap_cancel_s LDAP_P((
+	LDAP *ld,
+	int cancelid,
+	LDAPControl **sctrl,
+	LDAPControl **cctrl ));
 
 /*
  * in compare.c:
@@ -1077,7 +1113,6 @@ ldap_count_messages LDAP_P((
 	LDAP *ld,
 	LDAPMessage *chain ));
 
-
 /*
  * in references.c:
  */
@@ -1156,87 +1191,86 @@ typedef struct ldap_ava {
 	struct berval la_attr;
 	struct berval la_value;
 	unsigned la_flags;
-#define LDAP_AVA_STRING		0x0000U
-#define LDAP_AVA_BINARY		0x0001U
-#define LDAP_AVA_NONPRINTABLE	0x0002U
+#define LDAP_AVA_NULL				0x0000U
+#define LDAP_AVA_STRING				0x0001U
+#define LDAP_AVA_BINARY				0x0002U
+#define LDAP_AVA_NONPRINTABLE		0x0004U
+#define LDAP_AVA_FREE_ATTR		0x0010U
+#define LDAP_AVA_FREE_VALUE		0x0020U
 
 	void *la_private;
 } LDAPAVA;
 
 typedef LDAPAVA** LDAPRDN;
-typedef LDAPRDN** LDAPDN;
+typedef LDAPRDN* LDAPDN;
 
 /* DN formats */
-#define LDAP_DN_FORMAT_LDAP		0x0000U
+#define LDAP_DN_FORMAT_LDAP			0x0000U
 #define LDAP_DN_FORMAT_LDAPV3		0x0010U
 #define LDAP_DN_FORMAT_LDAPV2		0x0020U
-#define LDAP_DN_FORMAT_DCE		0x0030U
-#define LDAP_DN_FORMAT_UFN		0x0040U	/* dn2str only */
+#define LDAP_DN_FORMAT_DCE			0x0030U
+#define LDAP_DN_FORMAT_UFN			0x0040U	/* dn2str only */
 #define LDAP_DN_FORMAT_AD_CANONICAL	0x0050U	/* dn2str only */
-#define LDAP_DN_FORMAT_LBER		0x00F0U /* for testing only */
-#define LDAP_DN_FORMAT_MASK		0x00F0U
+#define LDAP_DN_FORMAT_LBER			0x00F0U /* for testing only */
+#define LDAP_DN_FORMAT_MASK			0x00F0U
 
 /* DN flags */
-#define LDAP_DN_PRETTY			0x0100U
-#define LDAP_DN_SKIP			0x0200U
+#define LDAP_DN_PRETTY				0x0100U
+#define LDAP_DN_SKIP				0x0200U
 #define LDAP_DN_P_NOLEADTRAILSPACES	0x1000U
 #define LDAP_DN_P_NOSPACEAFTERRDN	0x2000U
-#define LDAP_DN_PEDANTIC		0xF000U
+#define LDAP_DN_PEDANTIC			0xF000U
 
-LDAP_F( void )
-ldap_avafree LDAP_P(( LDAPAVA *ava ));
-LDAP_F( void )
-ldap_rdnfree LDAP_P(( LDAPRDN *rdn ));
-LDAP_F( void )
-ldap_dnfree LDAP_P(( LDAPDN *dn ));
+LDAP_F( void ) ldap_rdnfree LDAP_P(( LDAPRDN rdn ));
+LDAP_F( void ) ldap_dnfree LDAP_P(( LDAPDN dn ));
 
 LDAP_F( int )
 ldap_bv2dn LDAP_P(( 
 	struct berval *bv, 
-	LDAPDN **dn, 
+	LDAPDN *dn, 
 	unsigned flags ));
 
 LDAP_F( int )
 ldap_str2dn LDAP_P((
 	LDAP_CONST char *str,
-	LDAPDN **dn,
+	LDAPDN *dn,
 	unsigned flags ));
 
 LDAP_F( int )
 ldap_dn2bv LDAP_P((
-	LDAPDN *dn,
+	LDAPDN dn,
 	struct berval *bv,
 	unsigned flags ));
 
 LDAP_F( int )
 ldap_dn2str LDAP_P((
-	LDAPDN *dn,
+	LDAPDN dn,
 	char **str,
 	unsigned flags ));
 
 LDAP_F( int )
 ldap_bv2rdn LDAP_P((
 	struct berval *bv,
-	LDAPRDN **rdn,
+	LDAPRDN *rdn,
 	char **next,
 	unsigned flags ));
 
 LDAP_F( int )
 ldap_str2rdn LDAP_P((
 	LDAP_CONST char *str,
-	LDAPRDN **rdn,
+	LDAPRDN *rdn,
 	char **next,
 	unsigned flags ));
 
 LDAP_F( int )
 ldap_rdn2bv LDAP_P((
-	LDAPRDN *rdn,
+	LDAPRDN rdn,
 	struct berval *bv,
 	unsigned flags ));
 
 LDAP_F( int )
 ldap_rdn2str LDAP_P((
-	LDAPRDN *rdn,
+	LDAPRDN rdn,
 	char **str,
 	unsigned flags ));
 
@@ -1259,7 +1293,7 @@ ldap_explode_rdn LDAP_P(( /* deprecated */
 	LDAP_CONST char *rdn,
 	int notypes ));
 
-typedef int LDAPDN_rewrite_func LDAP_P(( LDAPDN *dn, unsigned flags ));
+typedef int LDAPDN_rewrite_func LDAP_P(( LDAPDN dn, unsigned flags, void *ctx ));
 
 LDAP_F( int )
 ldap_X509dn2bv LDAP_P(( void *x509_name, struct berval *dn,
@@ -1274,6 +1308,15 @@ ldap_dcedn2dn LDAP_P(( LDAP_CONST char *dce ));	/* deprecated */
 LDAP_F( char * )
 ldap_dn2ad_canonical LDAP_P(( LDAP_CONST char *dn ));	/* deprecated */
 
+LDAP_F( int )
+ldap_get_dn_ber LDAP_P((
+	LDAP *ld, LDAPMessage *e, BerElement **berout, struct berval *dn ));
+
+LDAP_F( int )
+ldap_get_attribute_ber LDAP_P((
+	LDAP *ld, LDAPMessage *e, BerElement *ber, struct berval *attr,
+	struct berval **vals ));
+
 /*
  * in getattr.c
  */
@@ -1435,23 +1478,12 @@ ldap_unbind_ext_s LDAP_P((
 	LDAPControl		**clientctrls));
 
 /*
- * in getfilter.c
- *	(deprecated)
+ * in filter.c
  */
-LDAP_F( LDAPFiltDesc * )
-ldap_init_getfilter LDAP_P(( /* deprecated */
-	LDAP_CONST char *fname ));
-
-LDAP_F( LDAPFiltInfo * )
-ldap_getfirstfilter LDAP_P(( /* deprecated */
-	LDAPFiltDesc *lfdp,
-	/* LDAP_CONST */ char *tagpat,
-	/* LDAP_CONST */ char *value ));
-
-LDAP_F( LDAPFiltInfo * )
-ldap_getnextfilter LDAP_P(( /* deprecated */
-	LDAPFiltDesc *lfdp ));
-
+LDAP_F( int )
+ldap_put_vrFilter LDAP_P((
+	BerElement *ber,
+	const char *vrf ));
 
 /*
  * in free.c
@@ -1483,10 +1515,6 @@ LDAP_F( char * )
 ldap_strdup LDAP_P((
 	LDAP_CONST char * ));
 
-LDAP_F( void )
-ldap_getfilter_free LDAP_P((
-	LDAPFiltDesc *lfdp ));
-
 LDAP_F( void )
 ldap_mods_free LDAP_P((
 	LDAPMod **mods,
@@ -1592,7 +1620,7 @@ ldap_parse_sort_control LDAP_P((
  */
 
 /*
- * structure for virtul list.
+ * structure for virtual list.
  */
 typedef struct ldapvlvinfo {
 	int             ldvlv_version;
@@ -1620,6 +1648,57 @@ ldap_parse_vlv_control LDAP_P((
 	struct berval **contextp,
 	int           *errcodep ));
 
+/*
+ * LDAP Who Am I? (whoami.c)
+ */
+
+LDAP_F( int )
+ldap_parse_whoami LDAP_P((
+	LDAP *ld,
+	LDAPMessage *res,
+	struct berval **authzid ));
+
+LDAP_F( int )
+ldap_whoami LDAP_P(( LDAP *ld,
+	LDAPControl		**sctrls,
+	LDAPControl		**cctrls,
+	int				*msgidp ));
+
+LDAP_F( int )
+ldap_whoami_s LDAP_P((
+	LDAP *ld,
+	struct berval **authzid,
+	LDAPControl **sctrls,
+	LDAPControl **cctrls ));
+
+/*
+ * in passwd.c
+ */
+
+LDAP_F( int )
+ldap_parse_passwd LDAP_P((
+	LDAP *ld,
+	LDAPMessage *res,
+	struct berval *newpasswd ));
+
+LDAP_F( int )
+ldap_passwd LDAP_P(( LDAP *ld,
+	struct berval	*user,
+	struct berval	*oldpw,
+	struct berval	*newpw,
+	LDAPControl		**sctrls,
+	LDAPControl		**cctrls,
+	int				*msgidp ));
+
+LDAP_F( int )
+ldap_passwd_s LDAP_P((
+	LDAP *ld,
+	struct berval	*user,
+	struct berval	*oldpw,
+	struct berval	*newpw,
+	struct berval *newpasswd,
+	LDAPControl **sctrls,
+	LDAPControl **cctrls ));
 
 LDAP_END_DECL
 #endif /* _LDAP_H */
