@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/abandon.c,v 1.25.2.1 2002/04/15 15:21:46 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/abandon.c,v 1.31.2.1 2003/09/18 15:43:32 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*  Portions
@@ -9,6 +9,10 @@
  *
  *  abandon.c
  */
+/*
+ * Portions Copyright (C) The Internet Society (1997)
+ * ASN.1 fragments are from RFC 2251; see RFC for full legal notices.
+ */
 
 /*
  * An abandon request looks like this:
@@ -58,16 +62,23 @@ ldap_abandon_ext(
 {
 	int rc;
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "abandon", LDAP_LEVEL_ARGS, "ldap_abandon_ext %d\n", msgid ));
+	LDAP_LOG ( OPERATION, ARGS, "ldap_abandon_ext %d\n", msgid, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_abandon_ext %d\n", msgid, 0, 0 );
 #endif
 
 	/* check client controls */
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
 	rc = ldap_int_client_controls( ld, cctrls );
-	if( rc != LDAP_SUCCESS ) return rc;
+	if( rc == LDAP_SUCCESS )
+		rc = do_abandon( ld, msgid, msgid, sctrls, cctrls );
 
-	return do_abandon( ld, msgid, msgid, sctrls, cctrls );
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+#endif
+	return rc;
 }
 
 
@@ -86,7 +97,7 @@ int
 ldap_abandon( LDAP *ld, int msgid )
 {
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "abandon", LDAP_LEVEL_ARGS, "ldap_abandon %d\n", msgid ));
+	LDAP_LOG ( OPERATION, ARGS, "ldap_abandon %d\n", msgid, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_abandon %d\n", msgid, 0, 0 );
 #endif
@@ -110,8 +121,7 @@ do_abandon(
 	LDAPRequest	*lr;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "abandon", LDAP_LEVEL_ARGS, "do_abandon %d, msgid %d\n", 
-		origid, msgid ));
+	LDAP_LOG ( OPERATION, ARGS, "do_abandon %d, msgid %d\n", origid, msgid, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "do_abandon origid %d, msgid %d\n",
 		origid, msgid, 0 );
@@ -142,7 +152,17 @@ do_abandon(
 		}
 	}
 
-	if ( ldap_msgdelete( ld, msgid ) == 0 ) {
+/* ldap_msgdelete locks the res_mutex. Give up the req_mutex
+ * while we're in there.
+ */
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+#endif
+	err = ldap_msgdelete( ld, msgid );
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
+	if ( err == 0 ) {
 		ld->ld_errno = LDAP_SUCCESS;
 		return LDAP_SUCCESS;
 	}
@@ -160,6 +180,7 @@ do_abandon(
 			ld->ld_errno = LDAP_NO_MEMORY;
 
 		} else {
+			LDAP_NEXT_MSGID(ld, i);
 #ifdef LDAP_CONNECTIONLESS
 			if ( LDAP_IS_UDP(ld) ) {
 			    err = ber_write( ber, ld->ld_options.ldo_peer,
@@ -170,14 +191,14 @@ do_abandon(
 			    char *dn = ld->ld_options.ldo_cldapdn;
 			    if (!dn) dn = "";
 			    err = ber_printf( ber, "{isti",  /* '}' */
-				++ld->ld_msgid, dn,
+				i, dn,
 				LDAP_REQ_ABANDON, msgid );
 			} else
 #endif
 			{
 			    /* create a message to send */
 			    err = ber_printf( ber, "{iti",  /* '}' */
-				++ld->ld_msgid,
+				i,
 				LDAP_REQ_ABANDON, msgid );
 			}
 
@@ -225,7 +246,7 @@ do_abandon(
 	}
 
 	if ( lr != NULL ) {
-		if ( sendabandon ) {
+		if ( sendabandon || lr->lr_status == LDAP_REQST_WRITING ) {
 			ldap_free_connection( ld, lr->lr_conn, 0, 1 );
 		}
 		if ( origid == msgid ) {
