@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/charray.c,v 1.9 2002/01/04 20:17:38 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/charray.c,v 1.12 2003/04/29 14:34:37 hallvard Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /* charray.c - routines for dealing with char * arrays */
@@ -17,7 +17,7 @@
 int
 ldap_charray_add(
     char	***a,
-    char	*s
+    const char *s
 )
 {
 	int	n;
@@ -120,18 +120,20 @@ ldap_charray_free( char **a )
 int
 ldap_charray_inlist(
     char	**a,
-    char	*s
+    const char *s
 )
 {
 	int	i;
 
-	for ( i = 0; a[i] != NULL; i++ ) {
+	if( a == NULL ) return 0;
+
+	for ( i=0; a[i] != NULL; i++ ) {
 		if ( strcasecmp( s, a[i] ) == 0 ) {
-			return( 1 );
+			return 1;
 		}
 	}
 
-	return( 0 );
+	return 0;
 }
 
 char **
