@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/kbind.c,v 1.27.2.1 2002/04/15 15:21:47 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/kbind.c,v 1.33.2.1 2003/09/18 15:43:32 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*  Portions
@@ -9,6 +9,10 @@
  *
  *  kbind.c
  */
+/*
+ * Portions Copyright (C) The Internet Society (1997)
+ * ASN.1 fragments are from RFC 2251; see RFC for full legal notices.
+ */
 
 /*
  *	BindRequest ::= SEQUENCE {
@@ -45,7 +49,6 @@
 
 #include "ldap-int.h"
 
-
 /*
  * ldap_kerberos_bind1 - initiate a bind to the ldap server using
  * kerberos authentication.  The dn is supplied.  It is assumed the user
@@ -63,9 +66,10 @@ ldap_kerberos_bind1( LDAP *ld, LDAP_CONST char *dn )
 	char		*cred;
 	int		rc;
 	ber_len_t credlen;
+	ber_int_t	id;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "kbind", LDAP_LEVEL_ENTRY, "ldap_kerberos_bind1\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_kerberos_bind1\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_kerberos_bind1\n", 0, 0, 0 );
 #endif
@@ -89,8 +93,9 @@ ldap_kerberos_bind1( LDAP *ld, LDAP_CONST char *dn )
 		return( -1 );
 	}
 
+	LDAP_NEXT_MSGID( ld, id );
 	/* fill it in */
-	rc = ber_printf( ber, "{it{istoN}N}", ++ld->ld_msgid, LDAP_REQ_BIND,
+	rc = ber_printf( ber, "{it{istoN}N}", id, LDAP_REQ_BIND,
 	    ld->ld_version, dn, LDAP_AUTH_KRBV41, cred, credlen );
 
 	if ( rc == -1 ) {
@@ -102,14 +107,9 @@ ldap_kerberos_bind1( LDAP *ld, LDAP_CONST char *dn )
 
 	LDAP_FREE( cred );
 
-#ifndef LDAP_NOCACHE
-	if ( ld->ld_cache != NULL ) {
-		ldap_flush_cache( ld );
-	}
-#endif /* !LDAP_NOCACHE */
 
 	/* send the message */
-	return ( ldap_send_initial_request( ld, LDAP_REQ_BIND, dn, ber ));
+	return ( ldap_send_initial_request( ld, LDAP_REQ_BIND, dn, ber, id ));
 }
 
 int
@@ -119,7 +119,7 @@ ldap_kerberos_bind1_s( LDAP *ld, LDAP_CONST char *dn )
 	LDAPMessage	*res;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "kbind", LDAP_LEVEL_ENTRY, "ldap_kerberos_bind1_s\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_kerberos_bind1_s\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_kerberos_bind1_s\n", 0, 0, 0 );
 #endif
@@ -129,7 +129,7 @@ ldap_kerberos_bind1_s( LDAP *ld, LDAP_CONST char *dn )
 		return( ld->ld_errno );
 
 	/* wait for a result */
-	if ( ldap_result( ld, ld->ld_msgid, 1, (struct timeval *) 0, &res )
+	if ( ldap_result( ld, msgid, 1, (struct timeval *) 0, &res )
 	    == -1 ) {
 		return( ld->ld_errno );	/* ldap_result sets ld_errno */
 	}
@@ -154,9 +154,10 @@ ldap_kerberos_bind2( LDAP *ld, LDAP_CONST char *dn )
 	char		*cred;
 	int		rc;
 	ber_len_t credlen;
+	ber_int_t id;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "kbind", LDAP_LEVEL_ENTRY, "ldap_kerberos_bind2\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_kerberos_bind2\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_kerberos_bind2\n", 0, 0, 0 );
 #endif
@@ -180,11 +181,11 @@ ldap_kerberos_bind2( LDAP *ld, LDAP_CONST char *dn )
 		return( -1 );
 	}
 
+	LDAP_NEXT_MSGID( ld, id );
 	/* fill it in */
-	rc = ber_printf( ber, "{it{istoN}N}", ++ld->ld_msgid, LDAP_REQ_BIND,
+	rc = ber_printf( ber, "{it{istoN}N}", id, LDAP_REQ_BIND,
 	    ld->ld_version, dn, LDAP_AUTH_KRBV42, cred, credlen );
 
-
 	LDAP_FREE( cred );
 
 	if ( rc == -1 ) {
@@ -194,7 +195,7 @@ ldap_kerberos_bind2( LDAP *ld, LDAP_CONST char *dn )
 	}
 
 	/* send the message */
-	return ( ldap_send_initial_request( ld, LDAP_REQ_BIND, dn, ber ));
+	return ( ldap_send_initial_request( ld, LDAP_REQ_BIND, dn, ber, id ));
 }
 
 /* synchronous bind to DSA using kerberos */
@@ -205,7 +206,7 @@ ldap_kerberos_bind2_s( LDAP *ld, LDAP_CONST char *dn )
 	LDAPMessage	*res;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "kbind", LDAP_LEVEL_ENTRY, "ldap_kerberos_bind2_s\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_kerberos_bind2_s\n" , 0, 0, 0);
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_kerberos_bind2_s\n", 0, 0, 0 );
 #endif
@@ -215,7 +216,7 @@ ldap_kerberos_bind2_s( LDAP *ld, LDAP_CONST char *dn )
 		return( ld->ld_errno );
 
 	/* wait for a result */
-	if ( ldap_result( ld, ld->ld_msgid, 1, (struct timeval *) 0, &res )
+	if ( ldap_result( ld, msgid, 1, (struct timeval *) 0, &res )
 	    == -1 ) {
 		return( ld->ld_errno );	/* ldap_result sets ld_errno */
 	}
@@ -230,7 +231,7 @@ ldap_kerberos_bind_s( LDAP *ld, LDAP_CONST char *dn )
 	int	err;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "kbind", LDAP_LEVEL_ENTRY, "ldap_kerberos_bind_s\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_kerberos_bind_s\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_kerberos_bind_s\n", 0, 0, 0 );
 #endif
@@ -261,17 +262,16 @@ ldap_get_kerberosv4_credentials(
 	char		realm[REALM_SZ], *cred, *krbinstance;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "kbind", LDAP_LEVEL_ENTRY, 
-		"ldap_get_kerberosv4_credentials\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_get_kerberosv4_credentials\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_get_kerberosv4_credentials\n", 0, 0, 0 );
 #endif
 
 	if ( (err = krb_get_tf_realm( tkt_string(), realm )) != KSUCCESS ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "kbind", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"ldap_get_kerberosv4_credentials: krb_get_tf_realm failed: %s\n",
-			krb_err_txt[err] ));
+			krb_err_txt[err], 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "ldap_get_kerberosv4_credentials: "
 			"krb_get_tf_realm failed: %s\n", krb_err_txt[err], 0, 0 );
@@ -293,9 +293,9 @@ ldap_get_kerberosv4_credentials(
 	    != KSUCCESS )
 	{
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "kbind", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"ldap_get_kerberosv4_credentials: krb_mk_req failed: %s\n",
-			krb_err_txt[err] ));
+			krb_err_txt[err], 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "ldap_get_kerberosv4_credentials: "
 			"krb_mk_req failed (%s)\n", krb_err_txt[err], 0, 0 );
