@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/modify.c,v 1.12.2.1 2002/04/15 15:21:47 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/modify.c,v 1.16.2.1 2003/09/18 15:43:32 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*  Portions
@@ -9,6 +9,10 @@
  *
  *  modify.c
  */
+/*
+ * Portions Copyright (C) The Internet Society (1997)
+ * ASN.1 fragments are from RFC 2251; see RFC for full legal notices.
+ */
 
 #include "portable.h"
 
@@ -37,7 +41,9 @@
  * Example:
  *	LDAPMod	*mods[] = { 
  *			{ LDAP_MOD_ADD, "cn", { "babs jensen", "babs", 0 } },
- *			{ LDAP_MOD_REPLACE, "sn", { "jensen", 0 } },
+ *			{ LDAP_MOD_REPLACE, "sn", { "babs jensen", "babs", 0 } },
+ *			{ LDAP_MOD_DELETE, "ou", 0 },
+ *			{ LDAP_MOD_INCREMENT, "uidNumber, { "1", 0 } }
  *			0
  *		}
  *	rc=  ldap_modify_ext( ld, dn, mods, sctrls, cctrls, &msgid );
@@ -52,6 +58,7 @@ ldap_modify_ext( LDAP *ld,
 {
 	BerElement	*ber;
 	int		i, rc;
+	ber_int_t	id;
 
 	/*
 	 * A modify request looks like this:
@@ -60,8 +67,9 @@ ldap_modify_ext( LDAP *ld,
 	 *		modifications	SEQUENCE OF SEQUENCE {
 	 *			operation	ENUMERATED {
 	 *				add	(0),
-	 *				delete	(1),
-	 *				replace	(2)
+	 *				delete (1),
+	 *				replace	(2),
+	 *				increment (3) -- extension
 	 *			},
 	 *			modification	SEQUENCE {
 	 *				type	AttributeType,
@@ -72,7 +80,7 @@ ldap_modify_ext( LDAP *ld,
 	 */
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "modify", LDAP_LEVEL_ENTRY, "ldap_modify_ext\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_modify_ext\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_modify_ext\n", 0, 0, 0 );
 #endif
@@ -86,8 +94,9 @@ ldap_modify_ext( LDAP *ld,
 		return( LDAP_NO_MEMORY );
 	}
 
-	if ( ber_printf( ber, "{it{s{" /*}}}*/, ++ld->ld_msgid, LDAP_REQ_MODIFY, dn )
-	    == -1 ) {
+	LDAP_NEXT_MSGID( ld, id );
+	rc = ber_printf( ber, "{it{s{" /*}}}*/, id, LDAP_REQ_MODIFY, dn );
+	if ( rc == -1 ) {
 		ld->ld_errno = LDAP_ENCODING_ERROR;
 		ber_free( ber, 1 );
 		return( ld->ld_errno );
@@ -131,7 +140,7 @@ ldap_modify_ext( LDAP *ld,
 	}
 
 	/* send the message */
-	*msgidp = ldap_send_initial_request( ld, LDAP_REQ_MODIFY, dn, ber );
+	*msgidp = ldap_send_initial_request( ld, LDAP_REQ_MODIFY, dn, ber, id );
 	return( *msgidp < 0 ? ld->ld_errno : LDAP_SUCCESS );
 }
 
@@ -149,7 +158,9 @@ ldap_modify_ext( LDAP *ld,
  * Example:
  *	LDAPMod	*mods[] = { 
  *			{ LDAP_MOD_ADD, "cn", { "babs jensen", "babs", 0 } },
- *			{ LDAP_MOD_REPLACE, "sn", { "jensen", 0 } },
+ *			{ LDAP_MOD_REPLACE, "sn", { "babs jensen", "babs", 0 } },
+ *			{ LDAP_MOD_DELETE, "ou", 0 },
+ *			{ LDAP_MOD_INCREMENT, "uidNumber, { "1", 0 } }
  *			0
  *		}
  *	msgid = ldap_modify( ld, dn, mods );
@@ -160,7 +171,7 @@ ldap_modify( LDAP *ld, LDAP_CONST char *dn, LDAPMod **mods )
 	int rc, msgid;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "modify", LDAP_LEVEL_ENTRY, "ldap_modify\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_modify\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_modify\n", 0, 0, 0 );
 #endif
