@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/sasl.c,v 1.50.2.1 2003/09/18 15:43:32 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/sasl.c,v 1.50.2.2 2003/10/06 14:57:29 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -433,6 +433,7 @@ ldap_sasl_interactive_bind_s(
 	void *defaults )
 {
 	int rc;
+	char *smechs = NULL;
 
 #if defined( LDAP_R_COMPILE ) && defined( HAVE_CYRUS_SASL )
 	ldap_pvt_thread_mutex_lock( &ldap_int_sasl_mutex );
@@ -448,9 +449,14 @@ ldap_sasl_interactive_bind_s(
 		goto done;
 	} else
 #endif
-	if( mechs == NULL || *mechs == '\0' ) {
-		char *smechs;
 
+#ifdef HAVE_CYRUS_SASL
+	if( mechs == NULL || *mechs == '\0' ) {
+		mechs = ld->ld_options.ldo_def_sasl_mech;
+	}
+#endif
+		
+	if( mechs == NULL || *mechs == '\0' ) {
 		rc = ldap_pvt_sasl_getmechs( ld, &smechs );
 		if( rc != LDAP_SUCCESS ) {
 			goto done;
@@ -488,6 +494,7 @@ done:
 #if defined( LDAP_R_COMPILE ) && defined( HAVE_CYRUS_SASL )
 	ldap_pvt_thread_mutex_unlock( &ldap_int_sasl_mutex );
 #endif
+	if ( smechs ) LDAP_FREE( smechs );
 
 	return rc;
 }
