@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/search.c,v 1.51.2.1 2002/04/15 15:21:48 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/search.c,v 1.57.2.2 2003/09/18 15:43:32 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*  Portions
@@ -21,7 +21,7 @@
 #include <ac/time.h>
 
 #include "ldap-int.h"
-
+#include "ldap_log.h"
 
 /*
  * ldap_search_ext - initiate an ldap search operation.
@@ -60,9 +60,10 @@ ldap_search_ext(
 	int rc;
 	BerElement	*ber;
 	int timelimit;
+	ber_int_t id;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "search", LDAP_LEVEL_ENTRY, "ldap_search_ext\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_search_ext\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_search_ext\n", 0, 0, 0 );
 #endif
@@ -76,7 +77,7 @@ ldap_search_ext(
 
 	/*
 	 * if timeout is provided, both tv_sec and tv_usec must
-	 * be non-zero
+	 * not be zero
 	 */
 	if( timeout != NULL ) {
 		if( timeout->tv_sec == 0 && timeout->tv_usec == 0 ) {
@@ -92,26 +93,15 @@ ldap_search_ext(
 	}
 
 	ber = ldap_build_search_req( ld, base, scope, filter, attrs,
-	    attrsonly, sctrls, cctrls, timelimit, sizelimit ); 
+	    attrsonly, sctrls, cctrls, timelimit, sizelimit, &id ); 
 
 	if ( ber == NULL ) {
 		return ld->ld_errno;
 	}
 
-#ifndef LDAP_NOCACHE
-	if ( ld->ld_cache != NULL ) {
-		if ( ldap_check_cache( ld, LDAP_REQ_SEARCH, ber ) == 0 ) {
-			ber_free( ber, 1 );
-			ld->ld_errno = LDAP_SUCCESS;
-			*msgidp = ld->ld_msgid;
-			return ld->ld_errno;
-		}
-		ldap_add_request_to_cache( ld, LDAP_REQ_SEARCH, ber );
-	}
-#endif /* LDAP_NOCACHE */
 
 	/* send the message */
-	*msgidp = ldap_send_initial_request( ld, LDAP_REQ_SEARCH, base, ber );
+	*msgidp = ldap_send_initial_request( ld, LDAP_REQ_SEARCH, base, ber, id );
 
 	if( *msgidp < 0 )
 		return ld->ld_errno;
@@ -150,7 +140,7 @@ ldap_search_ext_s(
 		return( ld->ld_errno );
 	}
 
-	if( rc == LDAP_RES_SEARCH_REFERENCE || rc == LDAP_RES_EXTENDED_PARTIAL ) {
+	if( rc == LDAP_RES_SEARCH_REFERENCE || rc == LDAP_RES_INTERMEDIATE ) {
 		return( ld->ld_errno );
 	}
 
@@ -182,9 +172,10 @@ ldap_search(
 	char **attrs, int attrsonly )
 {
 	BerElement	*ber;
+	ber_int_t	id;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "search", LDAP_LEVEL_ENTRY, "ldap_search\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_search\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_search\n", 0, 0, 0 );
 #endif
@@ -193,25 +184,15 @@ ldap_search(
 	assert( LDAP_VALID( ld ) );
 
 	ber = ldap_build_search_req( ld, base, scope, filter, attrs,
-	    attrsonly, NULL, NULL, -1, -1 ); 
+	    attrsonly, NULL, NULL, -1, -1, &id ); 
 
 	if ( ber == NULL ) {
 		return( -1 );
 	}
 
-#ifndef LDAP_NOCACHE
-	if ( ld->ld_cache != NULL ) {
-		if ( ldap_check_cache( ld, LDAP_REQ_SEARCH, ber ) == 0 ) {
-			ber_free( ber, 1 );
-			ld->ld_errno = LDAP_SUCCESS;
-			return( ld->ld_msgid );
-		}
-		ldap_add_request_to_cache( ld, LDAP_REQ_SEARCH, ber );
-	}
-#endif /* LDAP_NOCACHE */
 
 	/* send the message */
-	return ( ldap_send_initial_request( ld, LDAP_REQ_SEARCH, base, ber ));
+	return ( ldap_send_initial_request( ld, LDAP_REQ_SEARCH, base, ber, id ));
 }
 
 
@@ -226,10 +207,11 @@ ldap_build_search_req(
 	LDAPControl **sctrls,
 	LDAPControl **cctrls,
 	ber_int_t timelimit,
-	ber_int_t sizelimit )
+	ber_int_t sizelimit,
+	ber_int_t *idp)
 {
 	BerElement	*ber;
-	int		err;
+	int		err, id;
 
 	/*
 	 * Create the search request.  It looks like this:
@@ -270,6 +252,7 @@ ldap_build_search_req(
 		}
 	}
 
+	LDAP_NEXT_MSGID( ld, *idp );
 #ifdef LDAP_CONNECTIONLESS
 	if ( LDAP_IS_UDP(ld) ) {
 	    err = ber_write( ber, ld->ld_options.ldo_peer,
@@ -278,7 +261,7 @@ ldap_build_search_req(
 	if ( LDAP_IS_UDP(ld) && ld->ld_options.ldo_version == LDAP_VERSION2) {
 	    char *dn = ld->ld_options.ldo_cldapdn;
 	    if (!dn) dn = "";
-	    err = ber_printf( ber, "{ist{seeiib", ++ld->ld_msgid, dn,
+	    err = ber_printf( ber, "{ist{seeiib", *idp, dn,
 		LDAP_REQ_SEARCH, base, (ber_int_t) scope, ld->ld_deref,
 		(sizelimit < 0) ? ld->ld_sizelimit : sizelimit,
 		(timelimit < 0) ? ld->ld_timelimit : timelimit,
@@ -286,7 +269,7 @@ ldap_build_search_req(
 	} else
 #endif
 	{
-	    err = ber_printf( ber, "{it{seeiib", ++ld->ld_msgid,
+	    err = ber_printf( ber, "{it{seeiib", *idp,
 		LDAP_REQ_SEARCH, base, (ber_int_t) scope, ld->ld_deref,
 		(sizelimit < 0) ? ld->ld_sizelimit : sizelimit,
 		(timelimit < 0) ? ld->ld_timelimit : timelimit,
