@@ -1,14 +1,13 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/getopt.c,v 1.8 2002/01/04 20:17:43 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/getopt.c,v 1.11.2.1 2003/09/18 15:43:33 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
-	getopt.c
-
-	modified public-domain AT&T getopt(3)
-	modified by Kurt Zeilenga for inclusion into OpenLDAP
-*/
+ * getopt.c
+ * modified public-domain AT&T getopt(3)
+ * modified by Kurt Zeilenga for inclusion into OpenLDAP
+ */
 
 #include "portable.h"
 
@@ -23,6 +22,8 @@
 #include <io.h>
 #endif
 
+#include "lutil.h"
+
 #ifndef STDERR_FILENO
 #define STDERR_FILENO 2
 #endif
@@ -32,20 +33,27 @@ int optind = 1;
 int optopt;
 char * optarg;
 
+#ifdef HAVE_EBCDIC
+extern int _trans_argv;
+#endif
+
 static void ERR (char * const argv[], const char * s, char c)
 {
-	char errbuf[2];
-
 #ifdef DF_TRACE_DEBUG
 printf("DF_TRACE_DEBUG: 	static void ERR () in getopt.c\n");
 #endif
 	if (opterr)
 	{
-		errbuf[0] = c;
-		errbuf[1] = '\n';
-		(void) write(STDERR_FILENO,argv[0],strlen(argv[0]));
-		(void) write(STDERR_FILENO,s,strlen(s));
-		(void) write(STDERR_FILENO,errbuf,sizeof errbuf);
+		char *ptr, outbuf[4096];
+
+		ptr = lutil_strncopy(outbuf, argv[0], sizeof(outbuf) - 2);
+		ptr = lutil_strncopy(ptr, s, sizeof(outbuf)-2 -(ptr-outbuf));
+		*ptr++ = c;
+		*ptr++ = '\n';
+#ifdef HAVE_EBCDIC
+		__atoe_l(outbuf, ptr - outbuf);
+#endif
+		(void) write(STDERR_FILENO,outbuf,ptr - outbuf);
 	}
 }
 
@@ -57,6 +65,14 @@ int getopt (int argc, char * const argv [], const char * opts)
 
 #ifdef DF_TRACE_DEBUG
 printf("DF_TRACE_DEBUG: 	int getopt () in getopt.c\n");
+#endif
+
+#ifdef HAVE_EBCDIC
+	if (_trans_argv) {
+		int i;
+		for (i=0; i<argc; i++) __etoa(argv[i]);
+		_trans_argv = 0;
+	}
 #endif
 	if (sp == 1)
 	{
@@ -73,7 +89,7 @@ printf("DF_TRACE_DEBUG: 	int getopt () in getopt.c\n");
 	optopt = (int) c;
 	if (c == arg || (cp = strchr(opts,c)) == NULL)
 	{
-		ERR(argv,": illegal option--",c);
+		ERR(argv,_(": illegal option--"),c);
 		if (argv[optind][++sp] == eos)
 		{
 			optind++;
@@ -87,7 +103,7 @@ printf("DF_TRACE_DEBUG: 	int getopt () in getopt.c\n");
 			optarg = &argv[optind++][sp + 1];
 		else if (++optind >= argc)
 		{
-			ERR(argv,": option requires an argument--",c);
+			ERR(argv,_(": option requires an argument--"),c);
 			sp = 1;
 			return error;
 		}
