@@ -1,7 +1,7 @@
 /* aclparse.c - routines to parse and check acl's */
-/* $OpenLDAP: pkg/ldap/servers/slapd/aclparse.c,v 1.94.2.2 2002/04/18 15:19:58 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/aclparse.c,v 1.110.2.2 2003/09/22 04:09:39 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -17,6 +17,7 @@
 
 #include "slap.h"
 #include "lber_pvt.h"
+#include "lutil.h"
 
 static void		split(char *line, int splitchar, char **left, char **right);
 static void		access_append(Access **l, Access *a);
@@ -105,8 +106,8 @@ parse_acl(
 		/* to clause - select which entries are protected */
 		if ( strcasecmp( argv[i], "to" ) == 0 ) {
 			if ( a != NULL ) {
-				fprintf( stderr,
-		"%s: line %d: only one to clause allowed in access line\n",
+				fprintf( stderr, "%s: line %d: "
+					"only one to clause allowed in access line\n",
 				    fname, lineno );
 				acl_usage();
 			}
@@ -118,7 +119,9 @@ parse_acl(
 				}
 
 				if ( strcasecmp( argv[i], "*" ) == 0 ) {
-					if( a->acl_dn_pat.bv_len != 0 ) {
+					if( a->acl_dn_pat.bv_len ||
+						( a->acl_dn_style != ACL_STYLE_REGEX ) )
+					{
 						fprintf( stderr,
 							"%s: line %d: dn pattern"
 							" already specified in to clause.\n",
@@ -135,14 +138,16 @@ parse_acl(
 				split( left, '.', &left, &style );
 
 				if ( right == NULL ) {
-					fprintf( stderr,
-	"%s: line %d: missing \"=\" in \"%s\" in to clause\n",
+					fprintf( stderr, "%s: line %d: "
+						"missing \"=\" in \"%s\" in to clause\n",
 					    fname, lineno, left );
 					acl_usage();
 				}
 
 				if ( strcasecmp( left, "dn" ) == 0 ) {
-					if( a->acl_dn_pat.bv_len != 0 ) {
+					if( a->acl_dn_pat.bv_len != 0 ||
+						( a->acl_dn_style != ACL_STYLE_REGEX ) )
+					{
 						fprintf( stderr,
 							"%s: line %d: dn pattern"
 							" already specified in to clause.\n",
@@ -150,9 +155,34 @@ parse_acl(
 						acl_usage();
 					}
 
-					if ( style == NULL || *style == '\0'
-						|| strcasecmp( style, "regex" ) == 0 )
+					if ( style == NULL || *style == '\0' ||
+						( strcasecmp( style, "base" ) == 0 ) ||
+						( strcasecmp( style, "exact" ) == 0 ))
 					{
+						a->acl_dn_style = ACL_STYLE_BASE;
+						ber_str2bv( right, 0, 1, &a->acl_dn_pat );
+
+					} else if ( strcasecmp( style, "one" ) == 0 ) {
+						a->acl_dn_style = ACL_STYLE_ONE;
+						ber_str2bv( right, 0, 1, &a->acl_dn_pat );
+
+					} else if ( strcasecmp( style, "subtree" ) == 0
+						|| strcasecmp( style, "sub" ) == 0 )
+					{
+						if( *right == '\0' ) {
+							a->acl_dn_pat.bv_val = ch_strdup( "*" );
+							a->acl_dn_pat.bv_len = 1;
+
+						} else {
+							a->acl_dn_style = ACL_STYLE_SUBTREE;
+							ber_str2bv( right, 0, 1, &a->acl_dn_pat );
+						}
+
+					} else if ( strcasecmp( style, "children" ) == 0 ) {
+						a->acl_dn_style = ACL_STYLE_CHILDREN;
+						ber_str2bv( right, 0, 1, &a->acl_dn_pat );
+
+					} else if ( strcasecmp( style, "regex" ) == 0 ) {
 						a->acl_dn_style = ACL_STYLE_REGEX;
 
 						if ( *right == '\0' ) {
@@ -174,21 +204,10 @@ parse_acl(
 						} else {
 							acl_regex_normalized_dn( right, &a->acl_dn_pat );
 						}
-					} else if ( strcasecmp( style, "base" ) == 0 ) {
-						a->acl_dn_style = ACL_STYLE_BASE;
-						ber_str2bv( right, 0, 1, &a->acl_dn_pat );
-					} else if ( strcasecmp( style, "one" ) == 0 ) {
-						a->acl_dn_style = ACL_STYLE_ONE;
-						ber_str2bv( right, 0, 1, &a->acl_dn_pat );
-					} else if ( strcasecmp( style, "subtree" ) == 0 ) {
-						a->acl_dn_style = ACL_STYLE_SUBTREE;
-						ber_str2bv( right, 0, 1, &a->acl_dn_pat );
-					} else if ( strcasecmp( style, "children" ) == 0 ) {
-						a->acl_dn_style = ACL_STYLE_CHILDREN;
-						ber_str2bv( right, 0, 1, &a->acl_dn_pat );
+
 					} else {
-						fprintf( stderr,
-	"%s: line %d: unknown dn style \"%s\" in to clause\n",
+						fprintf( stderr, "%s: line %d: "
+							"unknown dn style \"%s\" in to clause\n",
 						    fname, lineno, style );
 						acl_usage();
 					}
@@ -197,8 +216,7 @@ parse_acl(
 				}
 
 				if ( strcasecmp( left, "filter" ) == 0 ) {
-					if ( (a->acl_filter = str2filter(
-					    right )) == NULL ) {
+					if ( (a->acl_filter = str2filter( right )) == NULL ) {
 						fprintf( stderr,
 				"%s: line %d: bad filter \"%s\" in to clause\n",
 						    fname, lineno, right );
@@ -214,6 +232,37 @@ parse_acl(
 						    fname, lineno, right );
 						acl_usage();
 					}
+
+				} else if ( strncasecmp( left, "val", 3 ) == 0 ) {
+					if ( a->acl_attrval.bv_len ) {
+						fprintf( stderr,
+				"%s: line %d: attr val already specified in to clause.\n",
+							fname, lineno );
+						acl_usage();
+					}
+					if ( a->acl_attrs == NULL || a->acl_attrs[1].an_name.bv_val ) {
+						fprintf( stderr,
+				"%s: line %d: attr val requires a single attribute.\n",
+							fname, lineno );
+						acl_usage();
+					}
+					ber_str2bv( right, 0, 1, &a->acl_attrval );
+					if ( style && strcasecmp( style, "regex" ) == 0 ) {
+						int e = regcomp( &a->acl_attrval_re, a->acl_attrval.bv_val,
+							REG_EXTENDED | REG_ICASE | REG_NOSUB );
+						if ( e ) {
+							char buf[512];
+							regerror( e, &a->acl_attrval_re, buf, sizeof(buf) );
+							fprintf( stderr, "%s: line %d: "
+								"regular expression \"%s\" bad because of %s\n",
+								fname, lineno, right, buf );
+							acl_usage();
+						}
+						a->acl_attrval_style = ACL_STYLE_REGEX;
+					} else {
+						a->acl_attrval_style = ACL_STYLE_BASE;
+					}
+					
 				} else {
 					fprintf( stderr,
 						"%s: line %d: expecting <what> got \"%s\"\n",
@@ -223,20 +272,22 @@ parse_acl(
 			}
 
 			if ( a->acl_dn_pat.bv_len != 0 &&
-				strcmp(a->acl_dn_pat.bv_val, "*") == 0)
+				strcmp(a->acl_dn_pat.bv_val, "*") == 0 )
 			{
 				free( a->acl_dn_pat.bv_val );
 				a->acl_dn_pat.bv_val = NULL;
 				a->acl_dn_pat.bv_len = 0;
 			}
 			
-			if( a->acl_dn_pat.bv_len != 0 ) {
+			if( a->acl_dn_pat.bv_len != 0 ||
+				( a->acl_dn_style != ACL_STYLE_REGEX ) )
+			{
 				if ( a->acl_dn_style != ACL_STYLE_REGEX ) {
 					struct berval bv;
-					rc = dnNormalize2( NULL, &a->acl_dn_pat, &bv);
+					rc = dnNormalize( 0, NULL, NULL, &a->acl_dn_pat, &bv, NULL);
 					if ( rc != LDAP_SUCCESS ) {
 						fprintf( stderr,
-							"%s: line %d: bad DN \"%s\"\n",
+							"%s: line %d: bad DN \"%s\" in to DN clause\n",
 							fname, lineno, a->acl_dn_pat.bv_val );
 						acl_usage();
 					}
@@ -259,8 +310,8 @@ parse_acl(
 		/* by clause - select who has what access to entries */
 		} else if ( strcasecmp( argv[i], "by" ) == 0 ) {
 			if ( a == NULL ) {
-				fprintf( stderr,
-					"%s: line %d: to clause required before by clause in access line\n",
+				fprintf( stderr, "%s: line %d: "
+					"to clause required before by clause in access line\n",
 				    fname, lineno );
 				acl_usage();
 			}
@@ -291,20 +342,27 @@ parse_acl(
 				if ( style ) {
 					split( style, ',', &style, &style_modifier);
 				}
-				if ( style == NULL || *style == '\0'
-					|| strcasecmp( style, "regex" ) == 0 )
+
+				if ( style == NULL || *style == '\0' ||
+					strcasecmp( style, "exact" ) == 0 ||
+					strcasecmp( style, "base" ) == 0 )
 				{
-					sty = ACL_STYLE_REGEX;
-				} else if ( strcasecmp( style, "exact" ) == 0 ) {
-					sty = ACL_STYLE_EXACT;
-				} else if ( strcasecmp( style, "base" ) == 0 ) {
 					sty = ACL_STYLE_BASE;
+
 				} else if ( strcasecmp( style, "one" ) == 0 ) {
 					sty = ACL_STYLE_ONE;
-				} else if ( strcasecmp( style, "subtree" ) == 0 ) {
+
+				} else if ( strcasecmp( style, "subtree" ) == 0 ||
+					strcasecmp( style, "sub" ) == 0 )
+				{
 					sty = ACL_STYLE_SUBTREE;
+
 				} else if ( strcasecmp( style, "children" ) == 0 ) {
 					sty = ACL_STYLE_CHILDREN;
+
+				} else if ( strcasecmp( style, "regex" ) == 0 ) {
+					sty = ACL_STYLE_REGEX;
+
 				} else {
 					fprintf( stderr,
 						"%s: line %d: unknown style \"%s\" in by clause\n",
@@ -312,28 +370,28 @@ parse_acl(
 					acl_usage();
 				}
 
-				if ( style_modifier && strcasecmp( style_modifier, "expand" ) == 0 ) {
+				if ( style_modifier &&
+					strcasecmp( style_modifier, "expand" ) == 0 )
+				{
 					expand = 1;
 				}
 
 				if ( strcasecmp( argv[i], "*" ) == 0 ) {
 					bv.bv_val = ch_strdup( "*" );
 					bv.bv_len = 1;
+					sty = ACL_STYLE_REGEX;
 
 				} else if ( strcasecmp( argv[i], "anonymous" ) == 0 ) {
-					ber_str2bv("anonymous",
-						sizeof("anonymous")-1,
-						1, &bv);
+					ber_str2bv("anonymous", sizeof("anonymous")-1, 1, &bv);
+					sty = ACL_STYLE_REGEX;
 
 				} else if ( strcasecmp( argv[i], "self" ) == 0 ) {
-					ber_str2bv("self",
-						sizeof("self")-1,
-						1, &bv);
+					ber_str2bv("self", sizeof("self")-1, 1, &bv);
+					sty = ACL_STYLE_REGEX;
 
 				} else if ( strcasecmp( argv[i], "users" ) == 0 ) {
-					ber_str2bv("users",
-						sizeof("users")-1,
-						1, &bv);
+					ber_str2bv("users", sizeof("users")-1, 1, &bv);
+					sty = ACL_STYLE_REGEX;
 
 				} else if ( strcasecmp( left, "dn" ) == 0 ) {
 					if ( sty == ACL_STYLE_REGEX ) {
@@ -382,8 +440,9 @@ parse_acl(
 							}
 						}
 					} else if ( right == NULL || *right == '\0' ) {
-						fprintf( stderr,
-							"%s: line %d: missing \"=\" in (or value after) \"%s\" in by clause\n",
+						fprintf( stderr, "%s: line %d: "
+							"missing \"=\" in (or value after) \"%s\" "
+							"in by clause\n",
 						    fname, lineno, left );
 						acl_usage();
 
@@ -404,10 +463,11 @@ parse_acl(
 					}
 
 					if ( sty != ACL_STYLE_REGEX && expand == 0 ) {
-						rc = dnNormalize2(NULL, &bv, &b->a_dn_pat);
+						rc = dnNormalize(0, NULL, NULL,
+							&bv, &b->a_dn_pat, NULL);
 						if ( rc != LDAP_SUCCESS ) {
 							fprintf( stderr,
-								"%s: line %d: bad DN \"%s\"\n",
+								"%s: line %d: bad DN \"%s\" in by DN clause\n",
 								fname, lineno, bv.bv_val );
 							acl_usage();
 						}
@@ -458,6 +518,14 @@ parse_acl(
 						acl_usage();
 					}
 
+					if( b->a_dn_at->ad_type->sat_equality == NULL ) {
+						fprintf( stderr,
+							"%s: line %d: dnattr \"%s\": "
+							"inappropriate matching (no EQUALITY)\n",
+							fname, lineno, right );
+						acl_usage();
+					}
+
 					continue;
 				}
 
@@ -466,15 +534,16 @@ parse_acl(
 					char *value = NULL;
 
 					if (sty != ACL_STYLE_REGEX && sty != ACL_STYLE_BASE) {
-						fprintf( stderr,
-							"%s: line %d: inappropriate style \"%s\" in by clause\n",
+						fprintf( stderr, "%s: line %d: "
+							"inappropriate style \"%s\" in by clause\n",
 						    fname, lineno, style );
 						acl_usage();
 					}
 
 					if ( right == NULL || right[ 0 ] == '\0' ) {
-						fprintf( stderr,
-							"%s: line %d: missing \"=\" in (or value after) \"%s\" in by clause\n",
+						fprintf( stderr, "%s: line %d: "
+							"missing \"=\" in (or value after) \"%s\" "
+							"in by clause\n",
 							fname, lineno, left );
 						acl_usage();
 					}
@@ -486,12 +555,11 @@ parse_acl(
 						acl_usage();
 					}
 
-					/* format of string is "group/objectClassValue/groupAttrName" */
+					/* format of string is
+						"group/objectClassValue/groupAttrName" */
 					if ((value = strchr(left, '/')) != NULL) {
 						*value++ = '\0';
-						if (*value
-							&& (name = strchr(value, '/')) != NULL)
-						{
+						if (*value && (name = strchr(value, '/')) != NULL) {
 							*name++ = '\0';
 						}
 					}
@@ -505,7 +573,8 @@ parse_acl(
 						b->a_group_pat = bv;
 					} else {
 						ber_str2bv( right, 0, 0, &bv );
-						rc = dnNormalize2( NULL, &bv, &b->a_group_pat );
+						rc = dnNormalize( 0, NULL, NULL, &bv,
+							&b->a_group_pat, NULL );
 						if ( rc != LDAP_SUCCESS ) {
 							fprintf( stderr,
 								"%s: line %d: bad DN \"%s\"\n",
@@ -581,7 +650,8 @@ parse_acl(
 					if( !is_at_syntax( b->a_group_at->ad_type,
 						SLAPD_DN_SYNTAX ) &&
 					    !is_at_syntax( b->a_group_at->ad_type,
-						SLAPD_NAMEUID_SYNTAX ) )
+						SLAPD_NAMEUID_SYNTAX ) &&
+						!is_at_subtype( b->a_group_at->ad_type, slap_schema.si_ad_labeledURI->ad_type ))
 					{
 						fprintf( stderr,
 							"%s: line %d: group \"%s\": inappropriate syntax: %s\n",
@@ -600,11 +670,12 @@ parse_acl(
 						vals[1].bv_val = NULL;
 
 
-						rc = oc_check_allowed( b->a_group_at->ad_type, vals, NULL );
+						rc = oc_check_allowed( b->a_group_at->ad_type,
+							vals, NULL );
 
 						if( rc != 0 ) {
-							fprintf( stderr,
-								"%s: line %d: group: \"%s\" not allowed by \"%s\"\n",
+							fprintf( stderr, "%s: line %d: "
+								"group: \"%s\" not allowed by \"%s\"\n",
 								fname, lineno,
 								b->a_group_at->ad_cname.bv_val,
 								b->a_group_oc->soc_oid );
@@ -616,22 +687,23 @@ parse_acl(
 
 				if ( strcasecmp( left, "peername" ) == 0 ) {
 					if (sty != ACL_STYLE_REGEX && sty != ACL_STYLE_BASE) {
-						fprintf( stderr,
-							"%s: line %d: inappropriate style \"%s\" in by clause\n",
+						fprintf( stderr, "%s: line %d: "
+							"inappropriate style \"%s\" in by clause\n",
 						    fname, lineno, style );
 						acl_usage();
 					}
 
 					if ( right == NULL || right[ 0 ] == '\0' ) {
-						fprintf( stderr,
-							"%s: line %d: missing \"=\" in (or value after) \"%s\" in by clause\n",
+						fprintf( stderr, "%s: line %d: "
+							"missing \"=\" in (or value after) \"%s\" "
+							"in by clause\n",
 							fname, lineno, left );
 						acl_usage();
 					}
 
 					if( b->a_peername_pat.bv_len ) {
-						fprintf( stderr,
-							"%s: line %d: peername pattern already specified.\n",
+						fprintf( stderr, "%s: line %d: "
+							"peername pattern already specified.\n",
 							fname, lineno );
 						acl_usage();
 					}
@@ -651,22 +723,23 @@ parse_acl(
 
 				if ( strcasecmp( left, "sockname" ) == 0 ) {
 					if (sty != ACL_STYLE_REGEX && sty != ACL_STYLE_BASE) {
-						fprintf( stderr,
-							"%s: line %d: inappropriate style \"%s\" in by clause\n",
+						fprintf( stderr, "%s: line %d: "
+							"inappropriate style \"%s\" in by clause\n",
 						    fname, lineno, style );
 						acl_usage();
 					}
 
 					if ( right == NULL || right[ 0 ] == '\0' ) {
-						fprintf( stderr,
-							"%s: line %d: missing \"=\" in (or value after) \"%s\" in by clause\n",
+						fprintf( stderr, "%s: line %d: "
+							"missing \"=\" in (or value after) \"%s\" "
+							"in by clause\n",
 							fname, lineno, left );
 						acl_usage();
 					}
 
 					if( b->a_sockname_pat.bv_len ) {
-						fprintf( stderr,
-							"%s: line %d: sockname pattern already specified.\n",
+						fprintf( stderr, "%s: line %d: "
+							"sockname pattern already specified.\n",
 							fname, lineno );
 						acl_usage();
 					}
@@ -1101,24 +1174,24 @@ accessmask2str( slap_mask_t mask, char *buf )
 
 	if ( ACL_IS_LEVEL( mask ) ) {
 		if ( ACL_LVL_IS_NONE(mask) ) {
-			ptr = slap_strcopy( ptr, "none" );
+			ptr = lutil_strcopy( ptr, "none" );
 
 		} else if ( ACL_LVL_IS_AUTH(mask) ) {
-			ptr = slap_strcopy( ptr, "auth" );
+			ptr = lutil_strcopy( ptr, "auth" );
 
 		} else if ( ACL_LVL_IS_COMPARE(mask) ) {
-			ptr = slap_strcopy( ptr, "compare" );
+			ptr = lutil_strcopy( ptr, "compare" );
 
 		} else if ( ACL_LVL_IS_SEARCH(mask) ) {
-			ptr = slap_strcopy( ptr, "search" );
+			ptr = lutil_strcopy( ptr, "search" );
 
 		} else if ( ACL_LVL_IS_READ(mask) ) {
-			ptr = slap_strcopy( ptr, "read" );
+			ptr = lutil_strcopy( ptr, "read" );
 
 		} else if ( ACL_LVL_IS_WRITE(mask) ) {
-			ptr = slap_strcopy( ptr, "write" );
+			ptr = lutil_strcopy( ptr, "write" );
 		} else {
-			ptr = slap_strcopy( ptr, "unknown" );
+			ptr = lutil_strcopy( ptr, "unknown" );
 		}
 		
 		*ptr++ = '(';
@@ -1252,29 +1325,28 @@ str2accessmask( const char *str )
 static void
 acl_usage( void )
 {
-	fprintf( stderr, "\n"
+	fprintf( stderr, "%s%s\n",
 		"<access clause> ::= access to <what> "
 				"[ by <who> <access> [ <control> ] ]+ \n"
-		"<what> ::= * | [dn[.<dnstyle>]=<regex>] [filter=<ldapfilter>] [attrs=<attrlist>]\n"
-		"<attrlist> ::= <attr> | <attr> , <attrlist>\n"
+		"<what> ::= * | [dn[.<dnstyle>]=<DN>] [filter=<filter>] [attrs=<attrlist>]\n"
+		"<attrlist> ::= <attr> [val[.<style>]=<value>] | <attr> , <attrlist>\n"
 		"<attr> ::= <attrname> | entry | children\n"
-		"<who> ::= [ * | anonymous | users | self | dn[.<dnstyle>]=<regex> ]\n"
+		"<who> ::= [ * | anonymous | users | self | dn[.<dnstyle>]=<DN> ]\n"
 			"\t[dnattr=<attrname>]\n"
-			"\t[group[/<objectclass>[/<attrname>]][.<style>]=<regex>]\n"
-			"\t[peername[.<style>]=<regex>] [sockname[.<style>]=<regex>]\n"
-			"\t[domain[.<style>]=<regex>] [sockurl[.<style>]=<regex>]\n"
+			"\t[group[/<objectclass>[/<attrname>]][.<style>]=<group>]\n"
+			"\t[peername[.<style>]=<peer>] [sockname[.<style>]=<name>]\n",
+			"\t[domain[.<style>]=<domain>] [sockurl[.<style>]=<url>]\n"
 #ifdef SLAPD_ACI_ENABLED
 			"\t[aci=<attrname>]\n"
 #endif
 			"\t[ssf=<n>] [transport_ssf=<n>] [tls_ssf=<n>] [sasl_ssf=<n>]\n"
-		"<dnstyle> ::= regex | base | exact (alias of base) | one | sub | children\n"
-		"<style> ::= regex | base | exact (alias of base)\n"
-		"<groupflags> ::= R\n"
+		"<dnstyle> ::= base | exact | one | subtree | children | regex\n"
+		"<style> ::= regex | base | exact\n"
 		"<access> ::= [self]{<level>|<priv>}\n"
 		"<level> ::= none | auth | compare | search | read | write\n"
 		"<priv> ::= {=|+|-}{w|r|s|c|x}+\n"
 		"<control> ::= [ stop | continue | break ]\n"
-		);
+	);
 	exit( EXIT_FAILURE );
 }
 
@@ -1474,14 +1546,8 @@ str2access( const char *str )
 
 #ifdef LDAP_DEBUG
 
-static char *style_strings[5] = {
-			"regex",
-			"base",
-			"one",
-			"subtree",
-			"children"
-		};
-
+static char *style_strings[5] = { "regex",
+	"base", "one", "subtree", "children" };
 
 static void
 print_access( Access *b )
@@ -1516,7 +1582,8 @@ print_access( Access *b )
 				b->a_group_oc->soc_oclass.oc_oid );
 
 			if ( b->a_group_at ) {
-				fprintf( stderr, " attributeType: %s", b->a_group_at->ad_cname.bv_val );
+				fprintf( stderr, " attributeType: %s",
+					b->a_group_at->ad_cname.bv_val );
 			}
 		}
     }
@@ -1618,6 +1685,13 @@ print_acl( Backend *be, AccessControl *a )
 		fprintf(  stderr, "\n" );
 	}
 
+	if ( a->acl_attrval.bv_len != 0 ) {
+		to++;
+		fprintf( stderr, " val.%s=%s\n",
+			style_strings[a->acl_attrval_style], a->acl_attrval.bv_val );
+
+	}
+
 	if( !to ) {
 		fprintf( stderr, " *\n" );
 	}
