@@ -1,5 +1,5 @@
 /* add.c - ldap BerkeleyDB back-end add routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/add.c,v 1.76.2.2 2003/09/22 04:09:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/add.c,v 1.76.2.4 2003/10/13 01:57:03 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -37,7 +37,7 @@ bdb_add(Operation *op, SlapReply *rs )
 
 	Operation* ps_list;
 	int		rc;
-	EntryInfo	*suffix_ei;
+	EntryInfo	*suffix_ei = NULL;
 	Entry		*ctxcsn_e;
 	int			ctxcsn_added = 0;
 
@@ -102,8 +102,8 @@ retry:	/* transaction retry */
 			rs->sr_text = "internal error";
 			goto return_results;
 		}
-		bdb_trans_backoff( ++num_retries );
 		ldap_pvt_thread_yield();
+		bdb_trans_backoff( ++num_retries );
 	}
 
 	/* begin transaction */
@@ -135,8 +135,6 @@ retry:	/* transaction retry */
 	
 	/*
 	 * Get the parent dn and see if the corresponding entry exists.
-	 * If the parent does not exist, only allow the "root" user to
-	 * add the entry.
 	 */
 	if ( be_issuffix( op->o_bd, &op->oq_add.rs_e->e_nname ) ) {
 		pdn = slap_empty_bv;
@@ -285,68 +283,22 @@ retry:	/* transaction retry */
 	} else {
 		/*
 		 * no parent!
-		 *	must be adding entry at suffix or with parent ""
+		 *  if not attempting to add entry at suffix or with parent ""
 		 */
-		if ( !be_isroot( op->o_bd, &op->o_ndn )) {
-			if ( be_issuffix( op->o_bd, (struct berval *)&slap_empty_bv )
-				|| be_isupdate( op->o_bd, &op->o_ndn ) )
-			{
-				p = (Entry *)&slap_entry_root;
-
-				/* check parent for "children" acl */
-				rs->sr_err = access_allowed( op, p,
-					children, NULL, ACL_WRITE, NULL );
-
-				p = NULL;
-
-				if ( ! rs->sr_err ) {
-					switch( opinfo.boi_err ) {
-					case DB_LOCK_DEADLOCK:
-					case DB_LOCK_NOTGRANTED:
-						goto retry;
-					}
-
+		if (( !be_isroot( op->o_bd, &op->o_ndn ) || pdn.bv_len > 0 )
+			&& !is_entry_glue( op->oq_add.rs_e ))
+		{
 #ifdef NEW_LOGGING
-					LDAP_LOG ( OPERATION, DETAIL1, 
-						"bdb_add: no write access to parent\n", 0, 0, 0 );
+			LDAP_LOG ( OPERATION, DETAIL1, "bdb_add: %s denied\n", 
+				pdn.bv_len == 0 ? "suffix" : "entry at root", 0, 0 );
 #else
-					Debug( LDAP_DEBUG_TRACE,
-						"bdb_add: no write access to parent\n",
-						0, 0, 0 );
-#endif
-					rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
-					rs->sr_text = "no write access to parent";
-					goto return_results;
-				}
-			} else if ( !is_entry_glue( op->oq_add.rs_e )) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, DETAIL1, "bdb_add: %s denied\n", 
-					pdn.bv_len == 0 ? "suffix" : "entry at root", 0, 0 );
-#else
-				Debug( LDAP_DEBUG_TRACE, "bdb_add: %s denied\n",
-					pdn.bv_len == 0 ? "suffix" : "entry at root",
-					0, 0 );
-#endif
-				rs->sr_err = LDAP_NO_SUCH_OBJECT;
-				goto return_results;
-			}
-		}
-
-#ifdef BDB_SUBENTRIES
-		if( subentry ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"bdb_add: no parent, cannot add subentry\n", 0, 0, 0 );
-#else
-			Debug( LDAP_DEBUG_TRACE,
-				"bdb_add: no parent, cannot add subentry\n",
-				0, 0, 0 );
+			Debug( LDAP_DEBUG_TRACE, "bdb_add: %s denied\n",
+				pdn.bv_len == 0 ? "suffix" : "entry at root",
+				0, 0 );
 #endif
 			rs->sr_err = LDAP_NO_SUCH_OBJECT;
-			rs->sr_text = "no parent, cannot add subentry";
 			goto return_results;
 		}
-#endif
 	}
 
 	if ( get_assert( op ) &&
@@ -506,6 +458,7 @@ retry:	/* transaction retry */
 	} else {
 		char gid[DB_XIDDATASIZE];
 
+		memset( gid, 0, sizeof(gid) );
 		snprintf( gid, sizeof( gid ), "%s-%08lx-%08lx",
 			bdb_uuid.bv_val, (long) op->o_connid, (long) op->o_opid );
 
