@@ -1,5 +1,5 @@
 /* dbcache.c - manage cache of open databases */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dbcache.c,v 1.28.2.1 2003/09/18 15:43:36 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dbcache.c,v 1.28.2.2 2003/10/06 14:57:30 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -46,19 +46,14 @@ bdb_db_hash(
 int
 bdb_db_cache(
 	Backend	*be,
-	DB_TXN *tid,
 	const char *name,
 	DB **dbout )
 {
 	int i;
 	int rc;
-	int flags;
 	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
 	struct bdb_db_info *db;
 	char *file;
-	DBT lockobj;
-	DB_LOCK lock;
-	u_int32_t locker = 0;
 
 	*dbout = NULL;
 
@@ -69,33 +64,19 @@ bdb_db_cache(
 		}
 	}
 
-	lockobj.data = "bdb_db_cache";
-	lockobj.size = sizeof("bdb_db_cache");
-
-	if (tid) {
-		locker = TXN_ID( tid );
-	} else {
-#ifdef BDB_REUSE_LOCKERS
-#define	op	NULL	/* implicit arg in LOCK_ID */
-#endif
-		rc = LOCK_ID( bdb->bi_dbenv, &locker );
-		if (rc) return rc;
-	}
-	rc = LOCK_GET( bdb->bi_dbenv, locker, 0, &lockobj,
-		DB_LOCK_WRITE, &lock );
-	if (rc) return rc;
+	ldap_pvt_thread_mutex_lock( &bdb->bi_database_mutex );
 
 	/* check again! may have been added by another thread */
 	for( i=BDB_NDB; i < bdb->bi_ndatabases; i++ ) {
 		if( !strcmp( bdb->bi_databases[i]->bdi_name, name) ) {
 			*dbout = bdb->bi_databases[i]->bdi_db;
-			LOCK_PUT( bdb->bi_dbenv, &lock);
+			ldap_pvt_thread_mutex_unlock( &bdb->bi_database_mutex );
 			return 0;
 		}
 	}
 
 	if( i >= BDB_INDICES ) {
-		LOCK_PUT( bdb->bi_dbenv, &lock);
+		ldap_pvt_thread_mutex_unlock( &bdb->bi_database_mutex );
 		return -1;
 	}
 
@@ -114,7 +95,7 @@ bdb_db_cache(
 			"bdb_db_cache: db_create(%s) failed: %s (%d)\n",
 			bdb->bi_dbenv_home, db_strerror(rc), rc );
 #endif
-		LOCK_PUT( bdb->bi_dbenv, &lock);
+		ldap_pvt_thread_mutex_unlock( &bdb->bi_database_mutex );
 		return rc;
 	}
 
@@ -129,11 +110,9 @@ bdb_db_cache(
 #ifdef HAVE_EBCDIC
 	__atoe( file );
 #endif
-	flags = bdb->bi_db_opflags | DB_CREATE | DB_THREAD;
-	if ( !tid ) flags |= DB_AUTO_COMMIT;
-	rc = DB_OPEN( db->bdi_db, tid,
+	rc = DB_OPEN( db->bdi_db,
 		file, NULL /* name */,
-		DB_HASH, flags,
+		DB_HASH, bdb->bi_db_opflags | DB_CREATE | DB_THREAD,
 		bdb->bi_dbenv_mode );
 
 	ch_free( file );
@@ -148,7 +127,7 @@ bdb_db_cache(
 			"bdb_db_cache: db_open(%s) failed: %s (%d)\n",
 			name, db_strerror(rc), rc );
 #endif
-		LOCK_PUT( bdb->bi_dbenv, &lock);
+		ldap_pvt_thread_mutex_unlock( &bdb->bi_database_mutex );
 		return rc;
 	}
 
@@ -157,6 +136,6 @@ bdb_db_cache(
 
 	*dbout = db->bdi_db;
 
-	LOCK_PUT( bdb->bi_dbenv, &lock );
+	ldap_pvt_thread_mutex_unlock( &bdb->bi_database_mutex );
 	return 0;
 }
