@@ -1,5 +1,5 @@
 /* dn2id.c - routines to deal with the dn2id index */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2id.c,v 1.84.2.2 2003/09/22 14:34:39 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2id.c,v 1.84.2.4 2003/10/13 01:57:03 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -899,26 +899,15 @@ struct dn2id_cookie {
 	Operation *op;
 };
 
-/* Stuff for iterating over a bei_kids AVL tree and adding the
- * IDs to an IDL
- */
-struct apply_arg {
-	ID *idl;
-	EntryInfo **ei;
-};
-
 static int
 apply_func(
 	void *data,
 	void *arg )
 {
 	EntryInfo *ei = data;
-	struct apply_arg *ap = arg;
+	ID *idl = arg;
 
-	bdb_idl_insert( ap->idl, ei->bei_id );
-	if ( ap->ei ) {
-		*(ap->ei)++ = ei;
-	}
+	bdb_idl_insert( idl, ei->bei_id );
 	return 0;
 }
 
@@ -927,7 +916,6 @@ hdb_dn2idl_internal(
 	struct dn2id_cookie *cx
 )
 {
-	EntryInfo **eilist = NULL, **ptr;
 #ifdef SLAP_IDL_CACHE
 	if ( cx->bdb->bi_idl_cache_size ) {
 		cx->rc = bdb_idl_cache_get(cx->bdb, cx->db, &cx->key, cx->tmp);
@@ -941,13 +929,26 @@ hdb_dn2idl_internal(
 #endif
 	BDB_IDL_ZERO( cx->tmp );
 
+	if ( !cx->ei ) {
+		cx->ei = bdb_cache_find_info( cx->bdb, cx->id );
+		if ( !cx->ei ) {
+			cx->rc = DB_NOTFOUND;
+			goto saveit;
+		}
+	}
+
+	bdb_cache_entryinfo_lock( cx->ei );
+
 	/* If number of kids in the cache differs from on-disk, load
 	 * up all the kids from the database
 	 */
 	if ( cx->ei->bei_ckids+1 != cx->ei->bei_dkids ) {
 		EntryInfo ei;
+		db_recno_t dkids = cx->ei->bei_dkids;
 		ei.bei_parent = cx->ei;
 
+		bdb_cache_entryinfo_unlock( cx->ei );
+
 		cx->rc = cx->db->cursor( cx->db, NULL, &cx->dbc,
 			cx->bdb->bi_db_opflags );
 		if ( cx->rc ) return cx->rc;
@@ -959,27 +960,20 @@ hdb_dn2idl_internal(
 
 		/* The first item holds the parent ID. Ignore it. */
 		cx->rc = cx->dbc->c_get( cx->dbc, &cx->key, &cx->data, DB_SET );
-		if ( cx->rc == DB_NOTFOUND ) goto saveit;
-		if ( cx->rc ) return cx->rc;
+		if ( cx->rc ) {
+			cx->dbc->c_close( cx->dbc );
+			if ( cx->rc == DB_NOTFOUND ) goto saveit;
+			return cx->rc;
+		}
 
 		/* If the on-disk count is zero we've never checked it.
 		 * Count it now.
 		 */
-		if ( !cx->ei->bei_dkids ) {
-			db_recno_t dkids;
+		if ( !dkids ) {
 			cx->dbc->c_count( cx->dbc, &dkids, 0 );
 			cx->ei->bei_dkids = dkids;
 		}
 
-		/* If there are kids and this is a subtree search, allocate
-		 * temp storage for the list of kids.
-		 */
-		if ( cx->prefix == DN_SUBTREE_PREFIX && cx->ei->bei_dkids > 1 ) {
-			eilist = cx->op->o_tmpalloc( sizeof(EntryInfo *) * cx->ei->bei_dkids, cx->op->o_tmpmemctx );
-			eilist[cx->ei->bei_dkids-1] = NULL;
-			ptr = eilist;
-		}
-
 		cx->data.data = cx->buf;
 		cx->data.ulen = BDB_IDL_UM_SIZE * sizeof(ID);
 		cx->data.flags = DB_DBT_USERMEM;
@@ -995,9 +989,11 @@ hdb_dn2idl_internal(
 				if (j) {
 					EntryInfo *ei2;
 					diskNode *d = (diskNode *)j;
+					short nrlen;
 
 					AC_MEMCPY( &ei.bei_id, &d->entryID, sizeof(ID) );
-					AC_MEMCPY( &ei.bei_nrdn.bv_len, &d->nrdnlen, sizeof(d->nrdnlen) );
+					AC_MEMCPY( &nrlen, &d->nrdnlen, sizeof(d->nrdnlen) );
+					ei.bei_nrdn.bv_len = nrlen;
 					/* nrdn/rdn are set in-place.
 					 * hdb_cache_load will copy them as needed
 					 */
@@ -1006,39 +1002,23 @@ hdb_dn2idl_internal(
 					ei.bei_rdn.bv_val = d->nrdn + ei.bei_nrdn.bv_len + 1;
 					bdb_idl_insert( cx->tmp, ei.bei_id );
 					hdb_cache_load( cx->bdb, &ei, &ei2 );
-					if ( eilist )
-						*ptr++ = ei2;
 				}
 			}
 		}
-		cx->dbc->c_close( cx->dbc );
+		cx->rc = cx->dbc->c_close( cx->dbc );
 	} else {
 		/* The in-memory cache is in sync with the on-disk data.
 		 * do we have any kids?
 		 */
+		cx->rc = 0;
 		if ( cx->ei->bei_ckids > 0 ) {
-			struct apply_arg ap;
-
-			/* Temp storage for subtree search */
-			if ( cx->prefix == DN_SUBTREE_PREFIX ) {
-				eilist = cx->op->o_tmpalloc( sizeof(EntryInfo *) * cx->ei->bei_dkids, cx->op->o_tmpmemctx );
-				eilist[cx->ei->bei_dkids-1] = NULL;
-			}
 
 			/* Walk the kids tree; order is irrelevant since bdb_idl_insert
 			 * will insert in sorted order.
 			 */
-			ap.idl = cx->tmp;
-			ap.ei = eilist;
-			bdb_cache_entryinfo_lock( cx->ei );
-			avl_apply( cx->ei->bei_kids, apply_func, &ap, -1, AVL_POSTORDER );
-			bdb_cache_entryinfo_unlock( cx->ei );
+			avl_apply( cx->ei->bei_kids, apply_func, cx->tmp, -1, AVL_POSTORDER );
 		}
-	}
-
-	/* If we got some records, treat as success */
-	if (!BDB_IDL_IS_ZERO(cx->tmp)) {
-		cx->rc = 0;
+		bdb_cache_entryinfo_unlock( cx->ei );
 	}
 
 saveit:
@@ -1049,16 +1029,34 @@ saveit:
 #endif
 	;
 gotit:
-	if ( cx->rc == 0 ) {
-		/* If eilist is NULL, cx->tmp is empty... */
-		if ( cx->prefix == DN_SUBTREE_PREFIX && eilist ) {
-			bdb_idl_union( cx->ids, cx->tmp );
-			for (ptr = eilist; *ptr; ptr++) {
-				cx->ei = *ptr;
-				cx->id = cx->ei->bei_id;
-				hdb_dn2idl_internal( cx );
+	if ( !BDB_IDL_IS_ZERO( cx->tmp )) {
+		if ( cx->prefix == DN_SUBTREE_PREFIX ) {
+			if (cx->ei->bei_state & CACHE_ENTRY_NO_GRANDKIDS) {
+				bdb_idl_union( cx->ids, cx->tmp );
+			} else {
+				ID *save, idcurs;
+				EntryInfo *ei = cx->ei;
+				int nokids = 1;
+				save = cx->op->o_tmpalloc( BDB_IDL_SIZEOF( cx->tmp ),
+					cx->op->o_tmpmemctx );
+				BDB_IDL_CPY( save, cx->tmp );
+				bdb_idl_union( cx->ids, cx->tmp );
+
+				idcurs = 0;
+				for ( cx->id = bdb_idl_first( save, &idcurs );
+					cx->id != NOID;
+					cx->id = bdb_idl_next( save, &idcurs )) {
+					cx->ei = NULL;
+					hdb_dn2idl_internal( cx );
+					if ( !BDB_IDL_IS_ZERO( cx->tmp ))
+						nokids = 0;
+				}
+				cx->op->o_tmpfree( save, cx->op->o_tmpmemctx );
+				if ( nokids ) ei->bei_state |= CACHE_ENTRY_NO_GRANDKIDS;
 			}
-			cx->op->o_tmpfree( eilist, cx->op->o_tmpmemctx );
+			/* Make sure caller knows it had kids! */
+			cx->tmp[0]=1;
+
 			cx->rc = 0;
 		} else {
 			BDB_IDL_CPY( cx->ids, cx->tmp );
