@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/proto-bdb.h,v 1.77.2.3 2003/09/22 14:34:39 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/proto-bdb.h,v 1.77.2.4 2003/10/06 14:57:30 kurt Exp $ */
 /*
  * Copyright 2000-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -55,7 +55,6 @@ int bdb_get_commit_csn LDAP_P(( Operation *op, SlapReply *rs,
 int
 bdb_db_cache(
     Backend	*be,
-    DB_TXN *tid,
     const char *name,
 	DB **db );
 
@@ -420,10 +419,12 @@ void bdb_unlocked_cache_return_entry_rw( Cache *cache, Entry *e, int rw );
 #define bdb_cache_delete			BDB_SYMBOL(cache_delete)
 #define bdb_cache_delete_cleanup		BDB_SYMBOL(cache_delete_cleanup)
 #define bdb_cache_find_id			BDB_SYMBOL(cache_find_id)
+#define bdb_cache_find_info			BDB_SYMBOL(cache_find_info)
 #define bdb_cache_find_ndn			BDB_SYMBOL(cache_find_ndn)
 #define bdb_cache_modify			BDB_SYMBOL(cache_modify)
 #define bdb_cache_modrdn			BDB_SYMBOL(cache_modrdn)
 #define bdb_cache_release_all		BDB_SYMBOL(cache_release_all)
+#define bdb_cache_delete_entry		BDB_SYMBOL(cache_delete_entry)
 
 int bdb_cache_children(
 	Operation *op,
@@ -459,6 +460,10 @@ int bdb_cache_find_ndn(
 	struct berval   *ndn,
 	EntryInfo	**res
 );
+EntryInfo * bdb_cache_find_info(
+	struct bdb_info *bdb,
+	ID id
+);
 int bdb_cache_find_id(
 	Operation *op,
 	DB_TXN	*tid,
@@ -476,9 +481,16 @@ int bdb_cache_delete(
 	DB_LOCK	*lock
 );
 void bdb_cache_delete_cleanup(
+	Cache	*cache,
 	Entry	*e
 );
 void bdb_cache_release_all( Cache *cache );
+void bdb_cache_delete_entry(
+	struct bdb_info *bdb,
+	EntryInfo *ei,
+	u_int32_t locker,
+	DB_LOCK *lock
+);
 
 #ifdef BDB_HIER
 int hdb_cache_load(
@@ -497,6 +509,10 @@ int bdb_cache_entry_db_relock(
 	int tryOnly,
 	DB_LOCK *lock );
 
+int bdb_cache_entry_db_unlock(
+	DB_ENV *env,
+	DB_LOCK *lock );
+
 #ifdef BDB_REUSE_LOCKERS
 
 #define bdb_locker_id				BDB_SYMBOL(locker_id)
