@@ -1,5 +1,5 @@
 /* search.c - search operation */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/search.c,v 1.136.2.2 2003/09/18 15:43:36 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/search.c,v 1.136.2.4 2003/10/13 01:57:03 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -1218,7 +1218,10 @@ loop_continue:
 			/* free reader lock */
 			if (!IS_PSEARCH) {
 				bdb_cache_return_entry_r( bdb->bi_dbenv,
-					&bdb->bi_cache, e , &lock);
+					&bdb->bi_cache, e , &lock );
+				if ( sop->o_nocaching ) {
+					bdb_cache_delete_entry( bdb, ei, locker, &lock );
+				}
 			}
 			e = NULL;
 			rs->sr_entry = NULL;
@@ -1512,8 +1515,8 @@ send_pagerequest_response(
 	int		tentries )
 {
 	LDAPControl	ctrl, *ctrls[2];
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement	*ber = (BerElement *)berbuf;
+	BerElementBuffer berbuf;
+	BerElement	*ber = (BerElement *)&berbuf;
 	struct berval	cookie = { 0, NULL };
 	PagedResultsCookie respcookie;
 
@@ -1576,8 +1579,8 @@ bdb_build_sync_state_ctrl(
 	int res;
 	const char *text = NULL;
 
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement *ber = (BerElement *)berbuf;
+	BerElementBuffer berbuf;
+	BerElement *ber = (BerElement *)&berbuf;
 
 	struct berval entryuuid_bv	= { 0, NULL };
 
@@ -1636,8 +1639,8 @@ bdb_build_sync_done_ctrl(
 	struct berval	*csn )
 {
 	int ret;
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement *ber = (BerElement *)berbuf;
+	BerElementBuffer berbuf;
+	BerElement *ber = (BerElement *)&berbuf;
 
 	ber_init2( ber, NULL, LBER_USE_DER );
 
@@ -1679,8 +1682,8 @@ bdb_send_ldap_intermediate(
 	int	    state,
 	struct berval *cookie )
 {
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement *ber = (BerElement *)berbuf;
+	BerElementBuffer berbuf;
+	BerElement *ber = (BerElement *)&berbuf;
 	struct berval rspdata;
 
 	int ret;
