@@ -1,5 +1,5 @@
 /* trans.c - bdb backend transaction routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/trans.c,v 1.1.2.1 2003/09/18 15:43:37 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/trans.c,v 1.1.2.2 2003/10/06 14:57:30 kurt Exp $ */
 
 #include "portable.h"
 
@@ -9,6 +9,7 @@
 #include "back-bdb.h"
 #include "external.h"
 #include "lber_pvt.h"
+#include "lutil.h"
 
 
 /* Congestion avoidance code
@@ -25,7 +26,7 @@ bdb_trans_backoff( int num_retries )
 	unsigned long max_key = -1;
 	struct timeval timeout;
 
-	lutil_entropy( &key, sizeof( unsigned long ));
+	lutil_entropy( (unsigned char *) &key, sizeof( unsigned long ));
 
 	for ( i = 0; i < num_retries; i++ ) {
 		if ( i >= 5 ) break;
