@@ -1,5 +1,5 @@
 /* add.c - ldap ldbm back-end add routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/add.c,v 1.73.2.1 2003/09/18 15:43:37 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/add.c,v 1.73.2.3 2003/10/13 01:57:03 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -99,8 +99,7 @@ ldbm_back_add(
 		dnParent( &op->o_req_ndn, &pdn );
 	}
 
-	if( pdn.bv_len )
-	{
+	if( pdn.bv_len ) {
 		Entry *matched = NULL;
 
 		/* get parent with writer lock */
@@ -134,7 +133,8 @@ ldbm_back_add(
 
 			ber_bvarray_free( rs->sr_ref );
 			free( (char *)rs->sr_matched );
-
+			rs->sr_ref = NULL;
+			rs->sr_matched = NULL;
 			return rs->sr_err;
 		}
 
@@ -190,7 +190,6 @@ ldbm_back_add(
 			    0, 0 );
 #endif
 
-
 			send_ldap_error( op, rs, LDAP_ALIAS_PROBLEM,
 			    "parent is an alias" );
 
@@ -220,6 +219,8 @@ ldbm_back_add(
 
 			ber_bvarray_free( rs->sr_ref );
 			free( (char *)rs->sr_matched );
+			rs->sr_ref = NULL;
+			rs->sr_matched = NULL;
 			return rs->sr_err;
 		}
 
@@ -231,76 +232,25 @@ ldbm_back_add(
 #endif
 
 	} else {
-		if( pdn.bv_val != NULL )
+		assert( pdn.bv_val == NULL || *pdn.bv_val == '\0' );
+
+		if ( !be_isroot( op->o_bd, &op->o_ndn )
+			&& !is_entry_glue( op->oq_add.rs_e ))
 		{
-			assert( *pdn.bv_val == '\0' );
+			ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
+
+#ifdef NEW_LOGGING
+			LDAP_LOG( BACK_LDBM, ERR,
+			   "ldbm_back_add: %s add denied.\n",
+			   pdn.bv_val == NULL ? "suffix" : "entry at root", 0, 0 );
+#else
+			Debug( LDAP_DEBUG_TRACE, "%s add denied\n",
+				pdn.bv_val == NULL ? "suffix" : "entry at root", 0, 0 );
+#endif
+
+			send_ldap_error( op, rs, LDAP_NO_SUCH_OBJECT, NULL );
+			return LDAP_NO_SUCH_OBJECT;
 		}
-
-		/* no parent, must be adding entry to root */
-		if ( !be_isroot( op->o_bd, &op->o_ndn ) )
-		{
-			if ( be_issuffix( op->o_bd, (struct berval *)&slap_empty_bv ) || be_isupdate( op->o_bd, &op->o_ndn ) ) {
-				p = (Entry *)&slap_entry_root;
-				
-				rs->sr_err = access_allowed( op, p,
-					children, NULL, ACL_WRITE, NULL );
-				p = NULL;
-				
-				if ( ! rs->sr_err ) {
-					ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
-
-#ifdef NEW_LOGGING
-					LDAP_LOG( BACK_LDBM, ERR,
-						"ldbm_back_add: No write "
-						"access to parent (\"\").\n", 0, 0, 0 );
-#else
-					Debug( LDAP_DEBUG_TRACE, 
-						"no write access to parent\n", 
-						0, 0, 0 );
-#endif
-
-					send_ldap_error( op, rs,
-						LDAP_INSUFFICIENT_ACCESS,
-						"no write access to parent" );
-
-					return LDAP_INSUFFICIENT_ACCESS;
-				}
-			} else if ( !is_entry_glue( op->oq_add.rs_e )) {
-				ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
-
-#ifdef NEW_LOGGING
-				LDAP_LOG( BACK_LDBM, ERR,
-					   "ldbm_back_add: %s add denied.\n",
-					   pdn.bv_val == NULL ? "suffix" 
-					   : "entry at root", 0, 0 );
-#else
-				Debug( LDAP_DEBUG_TRACE, "%s add denied\n",
-						pdn.bv_val == NULL ? "suffix" 
-						: "entry at root", 0, 0 );
-#endif
-
-				send_ldap_error( op, rs,
-						LDAP_NO_SUCH_OBJECT, NULL );
-
-					return LDAP_NO_SUCH_OBJECT;
-			}
-		}
-
-#ifdef LDBM_SUBENTRIES
-		        if( subentry ) {
-#ifdef NEW_LOGGING
-					LDAP_LOG ( OPERATION, DETAIL1,
-						"bdb_add: no parent, cannot add subentry\n", 0, 0, 0 );
-#else
-					Debug( LDAP_DEBUG_TRACE,
-						"bdb_add: no parent, cannot add subentry\n", 0, 0, 0 );
-#endif
-					rs->sr_err = LDAP_NO_SUCH_OBJECT;
-					rs->sr_text = "no parent, cannot add subentry";
-					goto return_results;
-				}
-#endif
-
 	}
 
 	if ( next_id( op->o_bd, &op->oq_add.rs_e->e_id ) ) {
@@ -408,6 +358,7 @@ ldbm_back_add(
 	}
 
 	rs->sr_err = LDAP_SUCCESS;
+	rs->sr_text = NULL;
 	send_ldap_result( op, rs );
 
 	/* marks the entry as committed, so it is added to the cache;
