@@ -1,5 +1,5 @@
 /* bind.c - ldbm backend bind and unbind routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/bind.c,v 1.65.2.1 2003/09/18 15:43:37 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/bind.c,v 1.65.2.2 2003/10/06 14:57:30 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -81,6 +81,8 @@ ldbm_back_bind(
 
 		if ( rs->sr_ref ) ber_bvarray_free( rs->sr_ref );
 		if ( rs->sr_matched ) free( (char *)rs->sr_matched );
+		rs->sr_ref = NULL;
+		rs->sr_matched = NULL;
 		return rs->sr_err;
 	}
 
@@ -145,7 +147,8 @@ ldbm_back_bind(
 		send_ldap_result( op, rs );
 
 		ber_bvarray_free( rs->sr_ref );
-
+		rs->sr_matched = NULL;
+		rs->sr_ref = NULL;
 		rc = rs->sr_err;
 		goto return_results;
 	}
