@@ -1,7 +1,7 @@
 /* cache.c - routines to maintain an in-core cache of entries */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/cache.c,v 1.57.2.1 2002/04/15 15:22:00 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/cache.c,v 1.61.2.1 2003/09/18 15:43:37 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -116,9 +116,9 @@ cache_return_entry_rw( Cache *cache, Entry *e, int rw )
 		ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
+		LDAP_LOG( CACHE, DETAIL1, 
 			   "cache_return_entry_rw: return (%ld):%s, refcnt=%d\n",
-			   id, rw ? "w" : "r", refcnt ));
+			   id, rw ? "w" : "r", refcnt );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"====> cache_return_entry_%s( %ld ): created (%d)\n",
@@ -131,9 +131,9 @@ cache_return_entry_rw( Cache *cache, Entry *e, int rw )
 			ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
+			LDAP_LOG( CACHE, DETAIL1, 
 				   "cache_return_entry_rw: %ld, delete pending (%d).\n",
-				   id, refcnt ));
+				   id, refcnt, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"====> cache_return_entry_%s( %ld ): delete pending (%d)\n",
@@ -150,9 +150,8 @@ cache_return_entry_rw( Cache *cache, Entry *e, int rw )
 			ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
-				   "cache_return_entry_rw: (%ld): deleted (%d)\n",
-				   id, refcnt ));
+			LDAP_LOG( CACHE, DETAIL1,
+				"cache_return_entry_rw: (%ld): deleted (%d)\n", id, refcnt, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"====> cache_return_entry_%s( %ld ): deleted (%d)\n",
@@ -165,9 +164,9 @@ cache_return_entry_rw( Cache *cache, Entry *e, int rw )
 		ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
+		LDAP_LOG( CACHE, DETAIL1, 
 			   "cache_return_entry_rw: ID %ld:%s returned (%d)\n",
-			   id, rw ? "w": "r", refcnt ));
+			   id, rw ? "w": "r", refcnt );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"====> cache_return_entry_%s( %ld ): returned (%d)\n",
@@ -218,9 +217,9 @@ cache_add_entry_rw(
 	Entry	*ee;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "cache", LDAP_LEVEL_ENTRY,
+	LDAP_LOG( CACHE, ENTRY, 
 		   "cache_add_entry_rw: add (%s):%s to cache\n",
-		   e->e_dn, rw ? "w" : "r" ));
+		   e->e_dn, rw ? "w" : "r", 0 );
 #endif
 	/* set cache mutex */
 	ldap_pvt_thread_mutex_lock( &cache->c_mutex );
@@ -232,9 +231,9 @@ cache_add_entry_rw(
 		ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_ERR,
+		LDAP_LOG( CACHE, ERR, 
 			   "cache_add_entry_rw: add (%s):%ld private init failed!\n",
-			   e->e_dn, e->e_id ));
+			   e->e_dn, e->e_id, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"====> cache_add_entry( %ld ): \"%s\": private init failed!\n",
@@ -245,15 +244,15 @@ cache_add_entry_rw(
 	}
 
 	if ( avl_insert( &cache->c_dntree, (caddr_t) e,
-		(AVL_CMP) entry_dn_cmp, avl_dup_error ) != 0 )
+	                 entry_dn_cmp, avl_dup_error ) != 0 )
 	{
 		/* free cache mutex */
 		ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
-			   "cache_add_entry: (%s):%ld already in cache.\n",
-			   e->e_dn, e->e_id ));
+		LDAP_LOG( CACHE, DETAIL1, 
+			"cache_add_entry: (%s):%ld already in cache.\n", 
+			e->e_dn, e->e_id, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"====> cache_add_entry( %ld ): \"%s\": already in dn cache\n",
@@ -267,12 +266,12 @@ cache_add_entry_rw(
 
 	/* id tree */
 	if ( avl_insert( &cache->c_idtree, (caddr_t) e,
-		(AVL_CMP) entry_id_cmp, avl_dup_error ) != 0 )
+	                 entry_id_cmp, avl_dup_error ) != 0 )
 	{
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
-			   "cache_add_entry: (%s):%ls already in cache.\n",
-			   e->e_dn, e->e_id ));
+		LDAP_LOG( CACHE, DETAIL1, 
+			"cache_add_entry: (%s):%ls already in cache.\n", 
+			e->e_dn, e->e_id, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"====> cache_add_entry( %ld ): \"%s\": already in id cache\n",
@@ -281,12 +280,12 @@ cache_add_entry_rw(
 
 		/* delete from dn tree inserted above */
 		if ( avl_delete( &cache->c_dntree, (caddr_t) e,
-			(AVL_CMP) entry_dn_cmp ) == NULL )
+		                 entry_dn_cmp ) == NULL )
 		{
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "cache", LDAP_LEVEL_INFO,
-				   "cache_add_entry: can't delete (%s) from cache.\n",
-				   e->e_dn ));
+			LDAP_LOG( CACHE, INFO, 
+				"cache_add_entry: can't delete (%s) from cache.\n", 
+				e->e_dn, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY, "====> can't delete from dn cache\n",
 			    0, 0, 0 );
@@ -369,12 +368,12 @@ cache_update_entry(
 	assert( e->e_private );
 
 	if ( avl_insert( &cache->c_dntree, (caddr_t) e,
-		(AVL_CMP) entry_dn_cmp, avl_dup_error ) != 0 )
+	                 entry_dn_cmp, avl_dup_error ) != 0 )
 	{
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
+		LDAP_LOG( CACHE, DETAIL1, 
 			   "cache_update_entry: (%s):%ld already in dn cache\n",
-			   e->e_dn, e->e_id ));
+			   e->e_dn, e->e_id, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"====> cache_update_entry( %ld ): \"%s\": already in dn cache\n",
@@ -388,12 +387,12 @@ cache_update_entry(
 
 	/* id tree */
 	if ( avl_insert( &cache->c_idtree, (caddr_t) e,
-		(AVL_CMP) entry_id_cmp, avl_dup_error ) != 0 )
+	                 entry_id_cmp, avl_dup_error ) != 0 )
 	{
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
+		LDAP_LOG( CACHE, DETAIL1, 
 			   "cache_update_entry: (%s)%ld already in id cache\n",
-			   e->e_dn, e->e_id ));
+			   e->e_dn, e->e_id, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"====> cache_update_entry( %ld ): \"%s\": already in id cache\n",
@@ -402,12 +401,12 @@ cache_update_entry(
 
 		/* delete from dn tree inserted above */
 		if ( avl_delete( &cache->c_dntree, (caddr_t) e,
-			(AVL_CMP) entry_dn_cmp ) == NULL )
+		                 entry_dn_cmp ) == NULL )
 		{
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "cache", LDAP_LEVEL_INFO,
-				   "cache_update_entry: can't delete (%s)%ld from dn cache.\n",
-				   e->e_dn, e->e_id ));
+			LDAP_LOG( CACHE, INFO, 
+				"cache_update_entry: can't delete (%s)%ld from dn cache.\n",
+				e->e_dn, e->e_id, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY, "====> can't delete from dn cache\n",
 			    0, 0, 0 );
@@ -485,7 +484,7 @@ try_again:
 	ldap_pvt_thread_mutex_lock( &cache->c_mutex );
 
 	if ( (ep = (Entry *) avl_find( cache->c_dntree, (caddr_t) &e,
-		(AVL_CMP) entry_dn_cmp )) != NULL )
+	                               entry_dn_cmp )) != NULL )
 	{
 		int state;
 		count++;
@@ -511,12 +510,12 @@ try_again:
 			ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "cache", LDAP_LEVEL_INFO,
-				   "cache_find_entry_dn2id: (%s) %ld not ready: %d\n",
-				   ndn->bv_val, id, state ));
+			LDAP_LOG( CACHE, INFO, 
+				   "cache_find_entry_ndn2id: (%s) %ld not ready: %d\n",
+				   ndn->bv_val, id, state );
 #else
 			Debug(LDAP_DEBUG_TRACE,
-				"====> cache_find_entry_dn2id(\"%s\"): %ld (not ready) %d\n",
+				"====> cache_find_entry_ndn2id(\"%s\"): %ld (not ready) %d\n",
 				ndn->bv_val, id, state);
 #endif
 
@@ -532,12 +531,12 @@ try_again:
 		ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
-			   "cache_find_entry_dn2id: (%s): %ld %d tries\n",
-			   ndn->bv_val, id, count ));
+		LDAP_LOG( CACHE, DETAIL1, 
+			   "cache_find_entry_ndn2id: (%s): %ld %d tries\n",
+			   ndn->bv_val, id, count );
 #else
 		Debug(LDAP_DEBUG_TRACE,
-			"====> cache_find_entry_dn2id(\"%s\"): %ld (%d tries)\n",
+			"====> cache_find_entry_ndn2id(\"%s\"): %ld (%d tries)\n",
 			ndn->bv_val, id, count);
 #endif
 
@@ -572,7 +571,7 @@ try_again:
 	ldap_pvt_thread_mutex_lock( &cache->c_mutex );
 
 	if ( (ep = (Entry *) avl_find( cache->c_idtree, (caddr_t) &e,
-		(AVL_CMP) entry_id_cmp )) != NULL )
+	                               entry_id_cmp )) != NULL )
 	{
 		int state;
 		ID	ep_id;
@@ -594,9 +593,9 @@ try_again:
 			ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "cache", LDAP_LEVEL_INFO,
+			LDAP_LOG( CACHE, INFO, 
 				   "cache_find_entry_id: (%ld)->%ld not ready (%d).\n",
-				   id, ep_id, state ));
+				   id, ep_id, state );
 				   
 #else
 			Debug(LDAP_DEBUG_TRACE,
@@ -618,9 +617,9 @@ try_again:
 		ldap_pvt_thread_mutex_unlock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
+		LDAP_LOG( CACHE, DETAIL1, 
 			   "cache_find_entry_id: %ld -> %s  found %d tries.\n",
-			   ep_id, ep->e_dn, count ));
+			   ep_id, ep->e_dn, count );
 #else
 		Debug(LDAP_DEBUG_TRACE,
 			"====> cache_find_entry_id( %ld ) \"%s\" (found) (%d tries)\n",
@@ -661,8 +660,8 @@ cache_delete_entry(
 	assert( e->e_private );
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "cache", LDAP_LEVEL_ENTRY,
-		   "cache_delete_entry: delete %ld.\n", e->e_id ));
+	LDAP_LOG( CACHE, ENTRY, 
+		"cache_delete_entry: delete %ld.\n", e->e_id, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "====> cache_delete_entry( %ld )\n",
 		e->e_id, 0, 0 );
@@ -684,15 +683,13 @@ cache_delete_entry_internal(
 	int rc = 0;	/* return code */
 
 	/* dn tree */
-	if ( avl_delete( &cache->c_dntree, (caddr_t) e, (AVL_CMP) entry_dn_cmp )
-		== NULL )
+	if ( avl_delete( &cache->c_dntree, (caddr_t) e, entry_dn_cmp ) == NULL )
 	{
 		rc = -1;
 	}
 
 	/* id tree */
-	if ( avl_delete( &cache->c_idtree, (caddr_t) e, (AVL_CMP) entry_id_cmp )
-		== NULL )
+	if ( avl_delete( &cache->c_idtree, (caddr_t) e, entry_id_cmp ) == NULL )
 	{
 		rc = -1;
 	}
@@ -723,8 +720,7 @@ cache_release_all( Cache *cache )
 	ldap_pvt_thread_mutex_lock( &cache->c_mutex );
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "cache", LDAP_LEVEL_ENTRY,
-		   "cache_release_all: enter\n" ));
+	LDAP_LOG( CACHE, ENTRY, "cache_release_all: enter\n" , 0, 0, 0);
 #else
 	Debug( LDAP_DEBUG_TRACE, "====> cache_release_all\n", 0, 0, 0 );
 #endif
@@ -740,8 +736,8 @@ cache_release_all( Cache *cache )
 
 	if ( cache->c_cursize ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_INFO,
-			   "cache_release_all: Entry cache could not be emptied.\n" ));
+		LDAP_LOG( CACHE, INFO,
+			 "cache_release_all: Entry cache could not be emptied.\n", 0, 0, 0);
 #else
 		Debug( LDAP_DEBUG_TRACE, "Entry-cache could not be emptied\n", 0, 0, 0 );
 #endif
