@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/external.h,v 1.37 2002/01/04 20:17:52 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/external.h,v 1.41 2003/03/26 10:50:04 hyc Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -37,12 +37,10 @@ extern BI_op_add	ldbm_back_add;
 
 extern BI_op_delete	ldbm_back_delete;
 
-extern BI_acl_group	ldbm_back_group;
-
-extern BI_acl_attribute	ldbm_back_attribute;
-
 extern BI_operational	ldbm_back_operational;
 
+extern BI_has_subordinates	ldbm_back_hasSubordinates;
+
 /* hooks for slap tools */
 extern BI_tool_entry_open	ldbm_tool_entry_open;
 extern BI_tool_entry_close	ldbm_tool_entry_close;
