@@ -1,7 +1,7 @@
 /* init.c - initialize ldbm backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/init.c,v 1.76.2.1 2002/04/15 15:22:01 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/init.c,v 1.84.2.1 2003/09/18 15:43:38 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -17,7 +17,7 @@
 
 #ifdef SLAPD_LDBM_DYNAMIC
 
-int back_ldbm_LTX_init_module(int argc, char *argv[]) {
+int init_module(int argc, char *argv[]) {
     BackendInfo bi;
 
     memset( &bi, '\0', sizeof(bi) );
@@ -37,6 +37,7 @@ ldbm_back_initialize(
 {
 	static char *controls[] = {
 		LDAP_CONTROL_MANAGEDSAIT,
+		LDAP_CONTROL_VALUESRETURNFILTER,
 		NULL
 	};
 
@@ -66,10 +67,10 @@ ldbm_back_initialize(
 	bi->bi_extended = ldbm_back_extended;
 
 	bi->bi_entry_release_rw = ldbm_back_entry_release_rw;
-	bi->bi_acl_group = ldbm_back_group;
-	bi->bi_acl_attribute = ldbm_back_attribute;
+	bi->bi_entry_get_rw = ldbm_back_entry_get;
 	bi->bi_chk_referrals = ldbm_back_referrals;
 	bi->bi_operational = ldbm_back_operational;
+	bi->bi_has_subordinates = ldbm_back_hasSubordinates;
 
 	/*
 	 * hooks for slap tools
@@ -83,6 +84,10 @@ ldbm_back_initialize(
 	bi->bi_tool_entry_reindex = ldbm_tool_entry_reindex;
 	bi->bi_tool_sync = ldbm_tool_sync;
 
+	bi->bi_tool_dn2id_get = 0;
+	bi->bi_tool_id2entry_get = 0;
+	bi->bi_tool_entry_modify = 0;
+
 	bi->bi_connection_init = 0;
 	bi->bi_connection_destroy = 0;
 
@@ -127,7 +132,13 @@ ldbm_back_db_init(
 	struct ldbminfo	*li;
 
 	/* indicate system schema supported */
-	be->be_flags |= SLAP_BFLAG_ALIASES|SLAP_BFLAG_REFERRALS;
+	be->be_flags |= 
+		SLAP_BFLAG_INCREMENT |
+#ifdef LDBM_SUBENTRIES
+		SLAP_BFLAG_SUBENTRIES |
+#endif
+		SLAP_BFLAG_ALIASES |
+		SLAP_BFLAG_REFERRALS;
 
 	/* allocate backend-database-specific stuff */
 	li = (struct ldbminfo *) ch_calloc( 1, sizeof(struct ldbminfo) );
@@ -201,8 +212,8 @@ ldbm_back_db_open(
 		if ( rc != 0 )
 		{
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "init", LDAP_LEVEL_ERR, "ldbm_back_db_open: sync "
-			"ldap_pvt_thread_create failed (%d)\n", rc ));
+			LDAP_LOG ( BACK_LDBM, ERR, "ldbm_back_db_open: sync "
+				"ldap_pvt_thread_create failed (%d)\n", rc, 0, 0 );
 #else	
 			Debug(	LDAP_DEBUG_ANY,
 				"sync ldap_pvt_thread_create failed (%d)\n", rc, 0, 0 );
