@@ -1,7 +1,7 @@
 /* tools.c - tools for slap tools */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/tools.c,v 1.34 2002/01/12 15:00:51 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/tools.c,v 1.39 2003/04/10 23:29:31 hyc Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -47,8 +47,8 @@ int ldbm_tool_entry_open(
 	if ( (id2entry = ldbm_cache_open( be, "id2entry", LDBM_SUFFIX, flags ))
 	    == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_CRIT,
-			   "Could not open/create id2entry%s\n", LDBM_SUFFIX ));
+		LDAP_LOG( BACK_LDBM, CRIT,
+			   "Could not open/create id2entry%s\n", LDBM_SUFFIX, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "Could not open/create id2entry" LDBM_SUFFIX "\n",
 		    0, 0, 0 );
@@ -173,13 +173,14 @@ ID ldbm_tool_entry_put(
 	Datum key, data;
 	int rc, len;
 	ID id;
+	Operation op = {0};
 
 	assert( slapMode & SLAP_TOOL_MODE );
 	assert( id2entry != NULL );
 
 	assert( text );
 	assert( text->bv_val );
-	assert( text->bv_val[0] == '\0' );
+	assert( text->bv_val[0] == '\0' );	/* overconservative? */
 
 	if ( next_id_get( be, &id ) || id == NOID ) {
 		strncpy( text->bv_val, "unable to get nextid", text->bv_len );
@@ -189,8 +190,8 @@ ID ldbm_tool_entry_put(
 	e->e_id = li->li_nextid++;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		"ldbm_tool_entry_put: (%s)%ld\n", e->e_dn, e->e_id ));
+	LDAP_LOG( BACK_LDBM, ENTRY,
+		"ldbm_tool_entry_put: (%s)%ld\n", e->e_dn, e->e_id ,0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "=> ldbm_tool_entry_put( %ld, \"%s\" )\n",
 		e->e_id, e->e_dn, 0 );
@@ -204,9 +205,9 @@ ID ldbm_tool_entry_put(
 
 	if( id != NOID ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
+		LDAP_LOG( BACK_LDBM, ENTRY,
 			"ldbm_tool_entry_put: \"%s\" already exists (id=%ld)\n",
-			e->e_dn, id ));
+			e->e_dn, id, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"<= ldbm_tool_entry_put: \"%s\" already exists (id=%ld)\n",
@@ -216,7 +217,11 @@ ID ldbm_tool_entry_put(
 		return NOID;
 	}
 
-	rc = index_entry_add( be, e, e->e_attrs );
+	op.o_bd = be;
+	op.o_tmpmemctx = NULL;
+	op.o_tmpmfuncs = &ch_mfuncs;
+
+	rc = index_entry_add( &op, e );
 	if( rc != 0 ) {
 		strncpy( text->bv_val, "index add failed", text->bv_len );
 		return NOID;
@@ -260,10 +265,11 @@ int ldbm_tool_entry_reindex(
 {
 	int rc;
 	Entry *e;
+	Operation op = {0};
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "ldbm_tool_entry_reindex: ID=%ld\n", (long)id ));
+	LDAP_LOG( BACK_LDBM, ENTRY, "ldbm_tool_entry_reindex: ID=%ld\n", 
+		(long)id, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ARGS, "=> ldbm_tool_entry_reindex( %ld )\n",
 		(long) id, 0, 0 );
@@ -274,9 +280,9 @@ int ldbm_tool_entry_reindex(
 
 	if( e == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
-			   "ldbm_tool_entry_reindex: could not locate id %ld\n", 
-			   (long)id ));
+		LDAP_LOG( BACK_LDBM, INFO,
+		   "ldbm_tool_entry_reindex: could not locate id %ld\n", 
+		   (long)id, 0, 0  );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"ldbm_tool_entry_reindex:: could not locate id=%ld\n",
@@ -294,15 +300,19 @@ int ldbm_tool_entry_reindex(
 	 */
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "ldbm_tool_entry_reindex: (%s) %ld\n", e->e_dn, id ));
+	LDAP_LOG( BACK_LDBM, ENTRY,
+		   "ldbm_tool_entry_reindex: (%s) %ld\n", e->e_dn, id, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "=> ldbm_tool_entry_reindex( %ld, \"%s\" )\n",
 		id, e->e_dn, 0 );
 #endif
 
 	dn2id_add( be, &e->e_nname, e->e_id );
-	rc = index_entry_add( be, e, e->e_attrs );
+
+	op.o_bd = be;
+	op.o_tmpmemctx = NULL;
+	op.o_tmpmfuncs = &ch_mfuncs;
+	rc = index_entry_add( &op, e );
 
 	entry_free( e );
 
