@@ -1,7 +1,7 @@
 /* init.c - initialize shell backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/init.c,v 1.23 2002/01/04 20:17:54 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/init.c,v 1.27.2.1 2003/09/18 15:43:39 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -16,7 +16,7 @@
 
 #ifdef SLAPD_SHELL_DYNAMIC
 
-int back_shell_LTX_init_module(int argc, char *argv[]) {
+int init_module(int argc, char *argv[]) {
     BackendInfo bi;
 
     memset( &bi, '\0', sizeof(bi) );
@@ -53,12 +53,10 @@ shell_back_initialize(
 	bi->bi_op_modrdn = shell_back_modrdn;
 	bi->bi_op_add = shell_back_add;
 	bi->bi_op_delete = shell_back_delete;
-	bi->bi_op_abandon = shell_back_abandon;
+	bi->bi_op_abandon = 0;
 
 	bi->bi_extended = 0;
 
-	bi->bi_acl_group = 0;
-	bi->bi_acl_attribute = 0;
 	bi->bi_chk_referrals = 0;
 
 	bi->bi_connection_init = 0;
