@@ -1,7 +1,7 @@
 /* unbind.c - shell backend unbind function */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/unbind.c,v 1.12 2002/01/04 20:17:55 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/unbind.c,v 1.18.2.1 2003/09/18 15:43:39 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -17,28 +17,26 @@
 
 int
 shell_back_unbind(
-    Backend		*be,
-    Connection		*conn,
-    Operation		*op
+    Operation		*op,
+    SlapReply		*rs
 )
 {
-	struct shellinfo	*si = (struct shellinfo *) be->be_private;
+	struct shellinfo	*si = (struct shellinfo *) op->o_bd->be_private;
 	FILE			*rfp, *wfp;
 
 	if ( si->si_unbind == NULL ) {
 		return 0;
 	}
 
-	if ( (op->o_private = (void *) forkandexec( si->si_unbind, &rfp, &wfp ))
-	    == (void *) -1 ) {
+	if ( forkandexec( si->si_unbind, &rfp, &wfp ) == (pid_t)-1 ) {
 		return 0;
 	}
 
 	/* write out the request to the unbind process */
 	fprintf( wfp, "UNBIND\n" );
 	fprintf( wfp, "msgid: %ld\n", (long) op->o_msgid );
-	print_suffixes( wfp, be );
-	fprintf( wfp, "dn: %s\n", (conn->c_dn.bv_len ? conn->c_dn.bv_val : "") );
+	print_suffixes( wfp, op->o_bd );
+	fprintf( wfp, "dn: %s\n", (op->o_conn->c_dn.bv_len ? op->o_conn->c_dn.bv_val : "") );
 	fclose( wfp );
 
 	/* no response to unbind */
