@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.206.2.2 2003/09/22 04:09:39 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.206.2.3 2003/10/06 14:57:29 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -336,6 +336,23 @@ int backend_startup(Backend *be)
 
 	/* open each backend database */
 	for( i = 0; i < nBackendDB; i++ ) {
+		if ( backendDB[i].be_update_ndn.bv_val && (
+			!backendDB[i].be_update_refs &&
+			!backendDB[i].syncinfo &&
+			!default_referral ) ) {
+#ifdef NEW_LOGGING
+			LDAP_LOG( BACKEND, CRIT, 
+				"backend_startup: slave \"%s\" updateref missing\n",
+				backendDB[i].be_suffix[0].bv_val, 0, 0 );
+				
+#else
+			Debug( LDAP_DEBUG_ANY,
+				"backend_startup: slave \"%s\" updateref missing\n",
+				backendDB[i].be_suffix[0].bv_val, 0, 0 );
+#endif
+			return -1;
+		}
+
 		/* append global access controls */
 		acl_append( &backendDB[i].be_acl, global_acl );
 
@@ -607,7 +624,8 @@ select_backend(
 				if( be == NULL ) {
 					be = &backends[i];
 
-					if( manageDSAit && len == dnlen ) {
+					if( manageDSAit && len == dnlen &&
+						!SLAP_GLUE_SUBORDINATE( be ) ) {
 						continue;
 					}
 				} else {
@@ -1177,6 +1195,7 @@ backend_group(
 
 	if ( target && dn_match( &target->e_nname, gr_ndn ) ) {
 		e = target;
+		rc = 0;
 	} else {
 		rc = be_entry_get_rw(op, gr_ndn, group_oc, group_at, 0, &e );
 	}
