@@ -1,5 +1,5 @@
 /* filter.c - routines for parsing and dealing with filters */
-/* $OpenLDAP: pkg/ldap/servers/slapd/filter.c,v 1.115 2003/04/29 18:40:26 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/filter.c,v 1.115.2.1 2003/10/06 14:57:29 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -26,6 +26,11 @@ static int	get_ssa(
 	SubstringsAssertion **s,
 	const char **text );
 
+static int filter_escape_value_x(
+	struct berval *in,
+	struct berval *out,
+	void *ctx );
+
 static void simple_vrFilter2bv(
 	Operation *op,
 	ValuesReturnFilter *f,
@@ -854,7 +859,7 @@ filter2bv( Filter *f, struct berval *fstr )
 	filter2bv_x( &op, f, fstr );
 }
 
-int
+static int
 filter_escape_value_x(
 	struct berval *in,
 	struct berval *out,
