@@ -1,7 +1,7 @@
 /* init.c - initialize various things */
-/* $OpenLDAP: pkg/ldap/servers/slapd/init.c,v 1.50.2.1 2002/04/15 15:21:54 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/init.c,v 1.59.2.1 2003/09/18 15:43:34 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -14,6 +14,10 @@
 #include <ac/time.h>
 
 #include "slap.h"
+#include "lber_pvt.h"
+#ifdef LDAP_SLAPI
+#include "slapi.h"
+#endif
 
 /*
  * read-only global variables or variables only written by the listener
@@ -33,6 +37,10 @@ int		ldap_syslog_level = LOG_DEBUG;
 
 BerVarray default_referral = NULL;
 
+struct berval AllUser = BER_BVC( LDAP_ALL_USER_ATTRIBUTES );
+struct berval AllOper = BER_BVC( LDAP_ALL_OPERATIONAL_ATTRIBUTES );
+struct berval NoAttrs = BER_BVC( LDAP_NO_ATTRS );
+
 /*
  * global variables that need mutex protection
  */
@@ -75,9 +83,9 @@ slap_init( int mode, const char *name )
 
 	if( slapMode != SLAP_UNDEFINED_MODE ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-			   "init: %s init called twice (old=%d, new=%d)\n",
-			   name, slapMode, mode ));
+		LDAP_LOG( OPERATION, CRIT, 
+			"init: %s init called twice (old=%d, new=%d)\n",
+			name, slapMode, mode );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		 "%s init: init called twice (old=%d, new=%d)\n",
@@ -93,9 +101,10 @@ slap_init( int mode, const char *name )
 		case SLAP_SERVER_MODE:
 		case SLAP_TOOL_MODE:
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1,
-				   "init: %s initiation, initiated %s.\n",
-				   name, (mode & SLAP_MODE) == SLAP_TOOL_MODE ? "tool" : "server" ));
+			LDAP_LOG( OPERATION, DETAIL1, 
+				"init: %s initiation, initiated %s.\n",
+				name, (mode & SLAP_MODE) == SLAP_TOOL_MODE ? 
+				  "tool" : "server", 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"%s init: initiated %s.\n",	name,
@@ -135,12 +144,13 @@ slap_init( int mode, const char *name )
 			if( rc == 0 ) {
 				rc = backend_init( );
 			}
+
 			break;
 
 		default:
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_ERR,
-				   "init: %s init, undefined mode (%d).\n", name, mode ));
+			LDAP_LOG( OPERATION, ERR, 
+				"init: %s init, undefined mode (%d).\n", name, mode, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"%s init: undefined mode (%d).\n", name, mode, 0 );
@@ -158,8 +168,7 @@ int slap_startup( Backend *be )
 	int rc;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-		   "slap_startup: %s started\n", slap_name ));
+	LDAP_LOG( OPERATION, CRIT, "slap_startup: %s started\n", slap_name, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"%s startup: initiated.\n",
@@ -169,16 +178,30 @@ int slap_startup( Backend *be )
 
 	rc = backend_startup( be );
 
+#ifdef LDAP_SLAPI
+	if( rc == 0 ) {
+		Slapi_PBlock *pb = slapi_pblock_new();
+
+		if ( doPluginFNs( NULL, SLAPI_PLUGIN_START_FN, pb ) < 0 ) {
+			rc = -1;
+		}
+		slapi_pblock_destroy( pb );
+	}
+#endif /* LDAP_SLAPI */
+
 	return rc;
 }
 
 int slap_shutdown( Backend *be )
 {
 	int rc;
+#ifdef LDAP_SLAPI
+	Slapi_PBlock *pb;
+#endif
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-		   "slap_shutdown: %s shutdown initiated.\n", slap_name));
+	LDAP_LOG( OPERATION, CRIT, 
+		"slap_shutdown: %s shutdown initiated.\n", slap_name, 0, 0);
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"%s shutdown: initiated\n",
@@ -191,6 +214,12 @@ int slap_shutdown( Backend *be )
 	/* let backends do whatever cleanup they need to do */
 	rc = backend_shutdown( be ); 
 
+#ifdef LDAP_SLAPI
+	pb = slapi_pblock_new( );
+	(void) doPluginFNs( NULL, SLAPI_PLUGIN_CLOSE_FN, pb );
+	slapi_pblock_destroy( pb );
+#endif /* LDAP_SLAPI */
+
 	return rc;
 }
 
@@ -199,9 +228,8 @@ int slap_destroy(void)
 	int rc;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-		   "slap_destroy: %s freeing system resources.\n",
-		   slap_name ));
+	LDAP_LOG( OPERATION, INFO, 
+		"slap_destroy: %s freeing system resources.\n", slap_name, 0, 0);
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"%s shutdown: freeing system resources.\n",
