@@ -1,5 +1,5 @@
 /* bind.c - ldbm backend bind and unbind routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/passwd.c,v 1.53.2.1 2003/09/18 15:43:34 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/passwd.c,v 1.53.2.2 2003/10/13 01:57:02 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -83,8 +83,8 @@ int slap_passwd_parse( struct berval *reqdata,
 	int rc = LDAP_SUCCESS;
 	ber_tag_t tag;
 	ber_len_t len;
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement *ber = (BerElement *)berbuf;
+	BerElementBuffer berbuf;
+	BerElement *ber = (BerElement *)&berbuf;
 
 	if( reqdata == NULL ) {
 		return LDAP_SUCCESS;
@@ -224,9 +224,9 @@ struct berval * slap_passwd_return(
 {
 	int rc;
 	struct berval *bv = NULL;
-	char berbuf[LBER_ELEMENT_SIZEOF];
+	BerElementBuffer berbuf;
 	/* opaque structure, size unknown but smaller than berbuf */
-	BerElement *ber = (BerElement *)berbuf;
+	BerElement *ber = (BerElement *)&berbuf;
 
 	assert( cred != NULL );
 
