@@ -1,5 +1,5 @@
 /* result.c - routines to send ldap results, errors, and referrals */
-/* $OpenLDAP: pkg/ldap/servers/slapd/result.c,v 1.186.2.2 2003/09/18 15:43:35 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/result.c,v 1.186.2.4 2003/10/13 01:57:03 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -223,8 +223,8 @@ send_ldap_response(
 	Operation *op,
 	SlapReply *rs )
 {
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement	*ber = (BerElement *)berbuf;
+	BerElementBuffer berbuf;
+	BerElement	*ber = (BerElement *) &berbuf;
 	int		rc;
 	long	bytes;
 
@@ -372,8 +372,8 @@ send_ldap_response(
 
 #ifdef LDAP_SLAPI
 	slapi_pblock_set( op->o_pb, SLAPI_RESULT_CODE, (void *)rs->sr_err );
-	slapi_pblock_set( op->o_pb, SLAPI_RESULT_MATCHED, ( rs->sr_matched != NULL ) ? (void *)ch_strdup( rs->sr_matched ) : NULL );
-	slapi_pblock_set( op->o_pb, SLAPI_RESULT_TEXT, ( rs->sr_text != NULL ) ? (void *)ch_strdup( rs->sr_text ) : NULL );
+	slapi_pblock_set( op->o_pb, SLAPI_RESULT_MATCHED, (void *)rs->sr_matched );
+	slapi_pblock_set( op->o_pb, SLAPI_RESULT_TEXT, (void *)rs->sr_text );
 #endif /* LDAP_SLAPI */
 
 	ldap_pvt_thread_mutex_lock( &num_sent_mutex );
@@ -487,6 +487,20 @@ slap_send_ldap_result( Operation *op, SlapReply *rs )
 		}
 	}
 
+#ifdef LDAP_SLAPI
+	/*
+	 * Call pre-result plugins. To avoid infinite recursion plugins
+	 * should just set SLAPI_RESULT_CODE rather than sending a
+	 * result if they wish to change the result.
+	 */
+	slapi_x_pblock_set_operation( op->o_pb, op );
+	slapi_pblock_set( op->o_pb, SLAPI_RESULT_CODE, (void *)rs->sr_err );
+	slapi_pblock_set( op->o_pb, SLAPI_RESULT_TEXT, (void *)rs->sr_text );
+	slapi_pblock_set( op->o_pb, SLAPI_RESULT_MATCHED, (void *)rs->sr_matched );
+
+	(void) doPluginFNs( op->o_bd, SLAPI_PLUGIN_PRE_RESULT_FN, op->o_pb );
+#endif /* LDAP_SLAPI */
+
 	if ( op->o_protocol < LDAP_VERSION3 ) {
 		tmp = v2ref( rs->sr_ref, rs->sr_text );
 		rs->sr_text = tmp;
@@ -588,8 +602,8 @@ slap_send_ldap_intermediate( Operation *op, SlapReply *rs )
 int
 slap_send_search_entry( Operation *op, SlapReply *rs )
 {
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement	*ber = (BerElement *)berbuf;
+	BerElementBuffer berbuf;
+	BerElement	*ber = (BerElement *) &berbuf;
 	Attribute	*a, *aa;
 	int		i, j, rc=-1, bytes;
 	char		*edn;
@@ -1179,8 +1193,8 @@ error_return:;
 int
 slap_send_search_reference( Operation *op, SlapReply *rs )
 {
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement	*ber = (BerElement *)berbuf;
+	BerElementBuffer berbuf;
+	BerElement	*ber = (BerElement *) &berbuf;
 	int rc = 0;
 	int bytes;
 	void *mark;
@@ -1423,8 +1437,8 @@ int slap_read_controls(
 {
 	int rc;
 	struct berval bv;
-	char berbuf[LBER_ELEMENT_SIZEOF];
-	BerElement *ber = (BerElement *) berbuf;
+	BerElementBuffer berbuf;
+	BerElement *ber = (BerElement *) &berbuf;
 	LDAPControl c;
 	ber_len_t	siz, len;
 	Operation myop;
