@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/root_dse.c,v 1.61.2.1 2002/04/15 15:21:56 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/root_dse.c,v 1.81.2.4 2003/09/22 04:09:39 kurt Exp $ */
 /* root_dse.c - Provides the ROOT DSA-Specific Entry
  *
- * Copyright 1999-2002 The OpenLDAP Foundation.
+ * Copyright 1999-2003 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted only
@@ -13,18 +13,26 @@
 #include "portable.h"
 
 #include <stdio.h>
+
 #include <ac/string.h>
 
 #include "slap.h"
 #include <ldif.h>
 #include "lber_pvt.h"
 
+#ifdef LDAP_SLAPI
+#include "slapi.h"
+#endif
+
 static struct berval supportedFeatures[] = {
-	BER_BVC("1.3.6.1.4.1.4203.1.5.1"), /* all Operational Attributes ("+") */
-	BER_BVC("1.3.6.1.4.1.4203.1.5.2"), /* OCs in Attributes List */
-	BER_BVC("1.3.6.1.4.1.4203.1.5.3"), /* (&) and (|) search filters */
-	BER_BVC("1.3.6.1.4.1.4203.1.5.4"), /* Language Tag Options */
-	BER_BVC("1.3.6.1.4.1.4203.1.5.5"), /* Language Range Options */
+	BER_BVC(LDAP_FEATURE_ALL_OPERATIONAL_ATTRS), /* All Op Attrs (+) */
+	BER_BVC(LDAP_FEATURE_OBJECTCLASS_ATTRS), /* OCs in Attrs List (+person) */
+	BER_BVC(LDAP_FEATURE_ABSOLUTE_FILTERS), /* (&) and (|) search filters */
+	BER_BVC(LDAP_FEATURE_LANGUAGE_TAG_OPTIONS), /* Language Tag Options */
+	BER_BVC(LDAP_FEATURE_LANGUAGE_RANGE_OPTIONS), /* Language Range Options */
+#ifdef LDAP_DEVEL
+	BER_BVC(LDAP_FEATURE_MODIFY_INCREMENT), /* Modify/increment */
+#endif
 	{0,NULL}
 };
 
@@ -36,9 +44,9 @@ root_dse_info(
 	Entry **entry,
 	const char **text )
 {
-	char buf[BUFSIZ];
 	Entry		*e;
-	struct berval	vals[2], *bv;
+	struct berval vals[2], *bv;
+	struct berval nvals[2];
 	int		i, j;
 	char ** supportedSASLMechanisms;
 
@@ -48,8 +56,6 @@ root_dse_info(
 		= slap_schema.si_ad_objectClass;
 	AttributeDescription *ad_namingContexts
 		= slap_schema.si_ad_namingContexts;
-	AttributeDescription *ad_supportedControl
-		= slap_schema.si_ad_supportedControl;
 	AttributeDescription *ad_supportedExtension
 		= slap_schema.si_ad_supportedExtension;
 	AttributeDescription *ad_supportedLDAPVersion
@@ -58,12 +64,26 @@ root_dse_info(
 		= slap_schema.si_ad_supportedSASLMechanisms;
 	AttributeDescription *ad_supportedFeatures
 		= slap_schema.si_ad_supportedFeatures;
+	AttributeDescription *ad_monitorContext
+		= slap_schema.si_ad_monitorContext;
 	AttributeDescription *ad_ref
 		= slap_schema.si_ad_ref;
 
 	vals[1].bv_val = NULL;
+	nvals[1].bv_val = NULL;
 
-	e = (Entry *) ch_calloc( 1, sizeof(Entry) );
+	e = (Entry *) SLAP_CALLOC( 1, sizeof(Entry) );
+
+	if( e == NULL ) {
+#ifdef NEW_LOGGING
+		LDAP_LOG( OPERATION, ERR,
+			"root_dse_info: SLAP_CALLOC failed", 0, 0, 0 );
+#else
+		Debug( LDAP_DEBUG_ANY,
+			"root_dse_info: SLAP_CALLOC failed", 0, 0, 0 );
+#endif
+		return LDAP_OTHER;
+	}
 
 	e->e_attrs = NULL;
 	e->e_name.bv_val = ch_strdup( LDAP_ROOT_DSE );
@@ -77,57 +97,84 @@ root_dse_info(
 
 	e->e_private = NULL;
 
-	vals[0].bv_val = "OpenLDAProotDSE";
-	vals[0].bv_len = sizeof("OpenLDAProotDSE")-1;
-	attr_merge( e, ad_structuralObjectClass, vals );
-
 	vals[0].bv_val = "top";
 	vals[0].bv_len = sizeof("top")-1;
-	attr_merge( e, ad_objectClass, vals );
+	if( attr_merge( e, ad_objectClass, vals, NULL ) ) {
+		return LDAP_OTHER;
+	}
 
 	vals[0].bv_val = "OpenLDAProotDSE";
 	vals[0].bv_len = sizeof("OpenLDAProotDSE")-1;
-	attr_merge( e, ad_objectClass, vals );
+	if( attr_merge( e, ad_objectClass, vals, NULL ) ) {
+		return LDAP_OTHER;
+	}
+	if( attr_merge( e, ad_structuralObjectClass, vals, NULL ) ) {
+		return LDAP_OTHER;
+	}
 
 	for ( i = 0; i < nbackends; i++ ) {
-		if ( backends[i].be_flags & SLAP_BFLAG_GLUE_SUBORDINATE ) {
+		if ( backends[i].be_flags & SLAP_BFLAG_MONITOR ) {
+			vals[0] = backends[i].be_suffix[0];
+			nvals[0] = backends[i].be_nsuffix[0];
+			if( attr_merge( e, ad_monitorContext, vals, nvals ) ) {
+				return LDAP_OTHER;
+			}
 			continue;
 		}
-		for ( j = 0; backends[i].be_suffix[j] != NULL; j++ ) {
-			vals[0] = *backends[i].be_suffix[j];
-			attr_merge( e, ad_namingContexts, vals );
+		if ( SLAP_GLUE_SUBORDINATE( &backends[i] ) ) {
+			continue;
+		}
+		for ( j = 0; backends[i].be_suffix[j].bv_val != NULL; j++ ) {
+			vals[0] = backends[i].be_suffix[j];
+			nvals[0] = backends[i].be_nsuffix[0];
+			if( attr_merge( e, ad_namingContexts, vals, nvals ) ) {
+				return LDAP_OTHER;
+			}
 		}
 	}
 
 	/* altServer unsupported */
 
 	/* supportedControl */
-	for ( i=0; (vals[0].bv_val = get_supported_ctrl(i)) != NULL; i++ ) {
-		vals[0].bv_len = strlen( vals[0].bv_val );
-		attr_merge( e, ad_supportedControl, vals );
+	if ( controls_root_dse_info( e ) != 0 ) {
+		return LDAP_OTHER;
 	}
 
 	/* supportedExtension */
-	for ( i=0; (bv = get_supported_extop(i)) != NULL; i++ ) {
-		vals[0] = *bv;
-		attr_merge( e, ad_supportedExtension, vals );
+	if ( exop_root_dse_info( e ) != 0 ) {
+		return LDAP_OTHER;
 	}
 
+#ifdef LDAP_SLAPI
+	/* netscape supportedExtension */
+	for ( i = 0; (bv = ns_get_supported_extop(i)) != NULL; i++ ) {
+		vals[0] = *bv;
+		if( attr_merge( e, ad_supportedExtension, vals, NULL )) {
+			return LDAP_OTHER;
+		}
+	}
+#endif /* LDAP_SLAPI */
+
 	/* supportedFeatures */
-	attr_merge( e, ad_supportedFeatures, supportedFeatures );
+	if( attr_merge( e, ad_supportedFeatures, supportedFeatures, NULL ) ) {
+		return LDAP_OTHER;
+	}
 
 	/* supportedLDAPVersion */
 	for ( i=LDAP_VERSION_MIN; i<=LDAP_VERSION_MAX; i++ ) {
+		char buf[BUFSIZ];
 		if (!( global_allows & SLAP_ALLOW_BIND_V2 ) &&
 			( i < LDAP_VERSION3 ) )
 		{
 			/* version 2 and lower are disallowed */
 			continue;
 		}
-		sprintf(buf,"%d",i);
+		snprintf(buf, sizeof buf, "%d", i);
 		vals[0].bv_val = buf;
 		vals[0].bv_len = strlen( vals[0].bv_val );
-		attr_merge( e, ad_supportedLDAPVersion, vals );
+		if( attr_merge( e, ad_supportedLDAPVersion, vals, NULL ) ) {
+			return LDAP_OTHER;
+		}
 	}
 
 	/* supportedSASLMechanism */
@@ -137,19 +184,27 @@ root_dse_info(
 		for ( i=0; supportedSASLMechanisms[i] != NULL; i++ ) {
 			vals[0].bv_val = supportedSASLMechanisms[i];
 			vals[0].bv_len = strlen( vals[0].bv_val );
-			attr_merge( e, ad_supportedSASLMechanisms, vals );
+			if( attr_merge( e, ad_supportedSASLMechanisms, vals, NULL ) ) {
+				return LDAP_OTHER;
+			}
 		}
-		charray_free( supportedSASLMechanisms );
+		ldap_charray_free( supportedSASLMechanisms );
 	}
 
 	if ( default_referral != NULL ) {
-		attr_merge( e, ad_ref, default_referral );
+		if( attr_merge( e, ad_ref, default_referral, NULL /* FIXME */ ) ) {
+			return LDAP_OTHER;
+		}
 	}
 
 	if( usr_attr != NULL) {
 		Attribute *a;
 		for( a = usr_attr->e_attrs; a != NULL; a = a->a_next ) {
-			attr_merge( e, a->a_desc, a->a_vals );
+			if( attr_merge( e, a->a_desc, a->a_vals,
+				(a->a_nvals == a->a_vals) ? NULL : a->a_nvals ) )
+			{
+				return LDAP_OTHER;
+			}
 		}
 	}
 
@@ -177,7 +232,18 @@ int read_root_dse_file( const char *fname )
 		return EXIT_FAILURE;
 	}
 
-	usr_attr = (Entry *) ch_calloc( 1, sizeof(Entry) );
+	usr_attr = (Entry *) SLAP_CALLOC( 1, sizeof(Entry) );
+	if( usr_attr == NULL ) {
+#ifdef NEW_LOGGING
+		LDAP_LOG( OPERATION, ERR,
+			"read_root_dse_file: SLAP_CALLOC failed", 0, 0, 0 );
+#else
+		Debug( LDAP_DEBUG_ANY,
+			"read_root_dse_file: SLAP_CALLOC failed", 0, 0, 0 );
+#endif
+		fclose( fp );
+		return LDAP_OTHER;
+	}
 	usr_attr->e_attrs = NULL;
 
 	while( ldif_read_record( fp, &lineno, &buf, &lmax ) ) {
@@ -187,10 +253,8 @@ int read_root_dse_file( const char *fname )
 		if( e == NULL ) {
 			fprintf( stderr, "root_dse: could not parse entry (line=%d)\n",
 				lineno );
-			entry_free( e );
-			entry_free( usr_attr );
-			usr_attr = NULL;
-			return EXIT_FAILURE;
+			rc = EXIT_FAILURE;
+			break;
 		}
 
 		/* make sure the DN is the empty DN */
@@ -199,9 +263,8 @@ int read_root_dse_file( const char *fname )
 				"root_dse: invalid rootDSE - dn=\"%s\" (line=%d)\n",
 				e->e_dn, lineno );
 			entry_free( e );
-			entry_free( usr_attr );
-			usr_attr = NULL;
-			return EXIT_FAILURE;
+			rc = EXIT_FAILURE;
+			break;
 		}
 
 		/*
@@ -211,14 +274,27 @@ int read_root_dse_file( const char *fname )
 		 */
 
 		for(a = e->e_attrs; a != NULL; a = a->a_next) {
-			attr_merge( usr_attr, a->a_desc, a->a_vals );
+			if( attr_merge( usr_attr, a->a_desc, a->a_vals,
+				(a->a_nvals == a->a_vals) ? NULL : a->a_nvals ) )
+			{
+				rc = LDAP_OTHER;
+				break;
+			}
 		}
 
 		entry_free( e );
+		if (rc) break;
+	}
+
+	if (rc) {
+		entry_free( usr_attr );
+		usr_attr = NULL;
 	}
 
 	ch_free( buf );
 
+	fclose( fp );
+
 	Debug(LDAP_DEBUG_CONFIG, "rootDSE file %s read.\n", fname, 0, 0);
 	return rc;
 }
