@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/tools/slapcommon.h,v 1.13.2.1 2003/09/18 15:43:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/tools/slapcommon.h,v 1.13.2.2 2003/10/13 01:57:03 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -18,7 +18,7 @@ enum slaptool {
 	SLAPTEST	/* database testing tool */
 };
 
-#define SLAP_TOOL_CTXCSN_NONE	0
+#define SLAP_TOOL_CTXCSN_KEEP	0
 #define SLAP_TOOL_CTXCSN_ENTRY	1
 #define SLAP_TOOL_CTXCSN_BATCH	2
 
@@ -30,6 +30,11 @@ extern	int		verbose;
 extern	int		update_ctxcsn;
 extern	int		retrieve_ctxcsn;
 extern	int		retrieve_synccookie;
+extern	int		replica_promotion;
+extern	int		replica_demotion;
+extern	char    *replica_id_string;
+extern	char    **replica_id_strlist;
+extern	int     *replica_id_list;
 extern	int		continuemode;
 extern	int		nosubordinates;
 extern	int		dryrun;
