@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/ldap_op.c,v 1.33.2.2 2002/04/18 15:20:03 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slurpd/ldap_op.c,v 1.43.2.1 2003/09/18 15:43:40 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -85,9 +85,10 @@ do_ldap(
 			lderr = op_ldap_add( ri, re, errmsg );
 			if ( lderr != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG (( " ldap_op", LDAP_LEVEL_ERR, "do_ldap: "
+				LDAP_LOG ( OPERATION, ERR, "do_ldap: "
 					"Error: ldap_add_s failed adding \"%s\": %s\n",
-					*errmsg ? *errmsg : ldap_err2string( lderr ), re->re_dn ));
+					*errmsg ? *errmsg : ldap_err2string( lderr ), 
+					re->re_dn, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
 					"Error: ldap_add_s failed adding \"%s\": %s\n",
@@ -101,9 +102,10 @@ do_ldap(
 			lderr = op_ldap_modify( ri, re, errmsg );
 			if ( lderr != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG (( " ldap_op", LDAP_LEVEL_ERR, "do_ldap: "
+				LDAP_LOG ( OPERATION, ERR, "do_ldap: "
 					"Error: ldap_modify_s failed modifying \"%s\": %s\n",
-					*errmsg ? *errmsg : ldap_err2string( lderr ), re->re_dn ));
+					*errmsg ? *errmsg : ldap_err2string( lderr ), 
+					re->re_dn, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
 					"Error: ldap_modify_s failed modifying \"%s\": %s\n",
@@ -117,9 +119,10 @@ do_ldap(
 			lderr = op_ldap_delete( ri, re, errmsg );
 			if ( lderr != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG (( " ldap_op", LDAP_LEVEL_ERR, "do_ldap: "
+				LDAP_LOG ( OPERATION, ERR, "do_ldap: "
 					"Error: ldap_delete_s failed deleting \"%s\": %s\n",
-					*errmsg ? *errmsg : ldap_err2string( lderr ), re->re_dn ));
+					*errmsg ? *errmsg : ldap_err2string( lderr ), 
+					re->re_dn, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
 					"Error: ldap_delete_s failed deleting \"%s\": %s\n",
@@ -133,9 +136,10 @@ do_ldap(
 			lderr = op_ldap_modrdn( ri, re, errmsg );
 			if ( lderr != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG (( " ldap_op", LDAP_LEVEL_ERR, "do_ldap: "
+				LDAP_LOG ( OPERATION, ERR, "do_ldap: "
 					"Error: ldap_modrdn_s failed modifying %s: %s\n",
-					*errmsg ? *errmsg : ldap_err2string( lderr ), re->re_dn ));
+					*errmsg ? *errmsg : ldap_err2string( lderr ), 
+					re->re_dn, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
 					"Error: ldap_modrdn_s failed modifying %s: %s\n",
@@ -147,9 +151,9 @@ do_ldap(
 
 		default:
 #ifdef NEW_LOGGING
-			LDAP_LOG (( " ldap_op", LDAP_LEVEL_ERR, "do_ldap: "
+			LDAP_LOG ( OPERATION, ERR, "do_ldap: "
 				"Error: bad op \"%d\", dn = \"%s\"\n",
-				re->re_changetype, re->re_dn ));
+				re->re_changetype, re->re_dn, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"Error: do_ldap: bad op \"%d\", dn = \"%s\"\n",
@@ -221,9 +225,9 @@ op_ldap_add(
 
 	/* Perform the operation */
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ARGS, 
+	LDAP_LOG ( OPERATION, ARGS, 
 		"op_ldap_add: replica %s:%d - add dn \"%s\"\n",
-		ri->ri_hostname, ri->ri_port, re->re_dn ));
+		ri->ri_hostname, ri->ri_port, re->re_dn );
 #else
 	Debug( LDAP_DEBUG_ARGS, "replica %s:%d - add dn \"%s\"\n",
 		ri->ri_hostname, ri->ri_port, re->re_dn );
@@ -235,8 +239,8 @@ op_ldap_add(
     } else {
 	*errmsg = "No modifications to do";
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
-		"op_ldap_add: Error: no mods to do (%s)!\n", re->re_dn ));
+	LDAP_LOG ( OPERATION, ERR, 
+		"op_ldap_add: Error: no mods to do (%s)!\n", re->re_dn, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY,
 	       "Error: op_ldap_add: no mods to do (%s)!\n", re->re_dn, 0, 0 );
@@ -277,8 +281,8 @@ op_ldap_modify(
     if ( re->re_mods == NULL ) {
 	*errmsg = "No arguments given";
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
-		"op_ldap_modify: Error: no arguments\n" ));
+	LDAP_LOG ( OPERATION, ERR, 
+		"op_ldap_modify: Error: no arguments\n" , 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY, "Error: op_ldap_modify: no arguments\n",
 		0, 0, 0 );
@@ -327,11 +331,21 @@ op_ldap_modify(
 	    nvals = 0;
 	    nops++;
 	    break;
+	case T_MODOPINCREMENT:
+	    state = T_MODOPINCREMENT;
+	    ldmarr = ( LDAPMod ** )
+		    ch_realloc(ldmarr, (( nops + 2 ) * ( sizeof( LDAPMod * ))));
+	    ldmarr[ nops ] = ldm = alloc_ldapmod();
+	    ldm->mod_op = LDAP_MOD_INCREMENT | LDAP_MOD_BVALUES;
+	    ldm->mod_type = value;
+	    nvals = 0;
+	    nops++;
+	    break;
 	default:
 	    if ( state == AWAITING_OP ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
-			"op_ldap_modify: Error: unknown mod type \"%s\"\n", type ));
+		LDAP_LOG ( OPERATION, ERR, 
+			"op_ldap_modify: Error: unknown mod type \"%s\"\n", type, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"Error: op_ldap_modify: unknown mod type \"%s\"\n",
@@ -348,10 +362,10 @@ op_ldap_modify(
 	     */
 	    if ( strcasecmp( type, ldm->mod_type )) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"op_ldap_modify: Error: "
 			"malformed modify op, %s: %s (expecting \"%s\")\n", 
-			type, value, ldm->mod_type ));
+			type, value, ldm->mod_type );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"Error: malformed modify op, %s: %s (expecting %s:)\n",
@@ -375,9 +389,9 @@ op_ldap_modify(
     if ( nops > 0 ) {
 	/* Actually perform the LDAP operation */
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_DETAIL1, 
+	LDAP_LOG ( OPERATION, DETAIL1, 
 		"op_ldap_modify: replica %s:%d - modify dn \"%s\"\n", 
-		ri->ri_hostname, ri->ri_port, re->re_dn ));
+		ri->ri_hostname, ri->ri_port, re->re_dn );
 #else
 	Debug( LDAP_DEBUG_ARGS, "replica %s:%d - modify dn \"%s\"\n",
 		ri->ri_hostname, ri->ri_port, re->re_dn );
@@ -404,9 +418,9 @@ op_ldap_delete(
     int		rc;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ARGS, 
+	LDAP_LOG ( OPERATION, ARGS, 
 		"op_ldap_delete: replica %s:%d - delete dn \"%s\"\n",
-	    ri->ri_hostname, ri->ri_port, re->re_dn ));
+	    ri->ri_hostname, ri->ri_port, re->re_dn );
 #else
     Debug( LDAP_DEBUG_ARGS, "replica %s:%d - delete dn \"%s\"\n",
 	    ri->ri_hostname, ri->ri_port, re->re_dn );
@@ -447,8 +461,8 @@ op_ldap_modrdn(
     if ( re->re_mods == NULL ) {
 	*errmsg = "No arguments given";
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
-		"op_ldap_modrdn: Error: no arguments\n" ));
+	LDAP_LOG ( OPERATION, ERR, 
+		"op_ldap_modrdn: Error: no arguments\n" , 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY, "Error: op_ldap_modrdn: no arguments\n",
 		0, 0, 0 );
@@ -463,9 +477,9 @@ op_ldap_modrdn(
 	if ( !strcmp( mi[ i ].mi_type, T_NEWRDNSTR )) {
 		if( state & GOT_NEWRDN ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"op_ldap_modrdn: Error: multiple newrdn arg \"%s\"\n",
-			mi[ i ].mi_val ));
+			mi[ i ].mi_val, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"Error: op_ldap_modrdn: multiple newrdn arg \"%s\"\n",
@@ -481,9 +495,9 @@ op_ldap_modrdn(
 	} else if ( !strcmp( mi[ i ].mi_type, T_DELOLDRDNSTR )) {
 		if( state & GOT_DELOLDRDN ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"op_ldap_modrdn: Error: multiple deleteoldrdn arg \"%s\"\n",
-			mi[ i ].mi_val ));
+			mi[ i ].mi_val, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"Error: op_ldap_modrdn: multiple deleteoldrdn arg \"%s\"\n",
@@ -500,9 +514,9 @@ op_ldap_modrdn(
 		drdnflag = 1;
 	    } else {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"op_ldap_modrdn: Error: bad deleteoldrdn arg \"%s\"\n",
-			mi[ i ].mi_val ));
+			mi[ i ].mi_val, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"Error: op_ldap_modrdn: bad deleteoldrdn arg \"%s\"\n",
@@ -515,9 +529,9 @@ op_ldap_modrdn(
 	} else if ( !strcmp( mi[ i ].mi_type, T_NEWSUPSTR )) {
 		if( state & GOT_NEWSUP ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"op_ldap_modrdn: Error: multiple newsuperior arg \"%s\"\n",
-			mi[ i ].mi_val ));
+			mi[ i ].mi_val, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"Error: op_ldap_modrdn: multiple newsuperior arg \"%s\"\n",
@@ -532,9 +546,9 @@ op_ldap_modrdn(
 
 	} else {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"op_ldap_modrdn: Error: bad type \"%s\"\n",
-			mi[ i ].mi_type ));
+			mi[ i ].mi_type, 0, 0 );
 #else
 	    Debug( LDAP_DEBUG_ANY, "Error: op_ldap_modrdn: bad type \"%s\"\n",
 		    mi[ i ].mi_type, 0, 0 );
@@ -549,8 +563,8 @@ op_ldap_modrdn(
      */
     if ( !GOT_ALL_MODDN(state) ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
-			"op_ldap_modrdn: Error: missing arguments\n" ));
+		LDAP_LOG ( OPERATION, ERR, 
+			"op_ldap_modrdn: Error: missing arguments\n" , 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY, "Error: op_ldap_modrdn: missing arguments\n",
 		0, 0, 0 );
@@ -563,14 +577,17 @@ op_ldap_modrdn(
     if ( ldap_debug & LDAP_DEBUG_ARGS ) {
 	char buf[ 256 ];
 	char *buf2;
-	sprintf( buf, "%s:%d", ri->ri_hostname, ri->ri_port );
-	buf2 = (char *) ch_malloc( strlen( re->re_dn ) + strlen( mi->mi_val )
-		+ 10 );
-	sprintf( buf2, "(\"%s\" -> \"%s\")", re->re_dn, mi->mi_val );
+	int buf2len = strlen( re->re_dn ) + strlen( mi->mi_val ) + 11;
+
+	snprintf( buf, sizeof(buf), "%s:%d", ri->ri_hostname, ri->ri_port );
+
+	buf2 = (char *) ch_malloc( buf2len );
+	snprintf( buf2, buf2len, "(\"%s\" -> \"%s\")", re->re_dn, mi->mi_val );
+
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ARGS, 
+	LDAP_LOG ( OPERATION, ARGS, 
 		"op_ldap_modrdn: replica %s - modify rdn %s (flag: %d)\n",
-		buf, buf2, drdnflag ));
+		buf, buf2, drdnflag );
 #else
 	Debug( LDAP_DEBUG_ARGS,
 		"replica %s - modify rdn %s (flag: %d)\n",
@@ -689,6 +706,9 @@ char *type )
     if ( !strcmp( type, T_MODOPDELETESTR )) {
 	return( T_MODOPDELETE );
     }
+    if ( !strcmp( type, T_MODOPINCREMENTSTR )) {
+	return( T_MODOPINCREMENT );
+    }
     return( T_ERR );
 }
 
@@ -709,9 +729,9 @@ do_unbind(
 	rc = ldap_unbind( ri->ri_ldp );
 	if ( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"do_unbind: ldap_unbind failed for %s:%d: %s\n",
-		    ri->ri_hostname, ri->ri_port, ldap_err2string( rc ) ));
+		    ri->ri_hostname, ri->ri_port, ldap_err2string( rc ) );
 #else
 	    Debug( LDAP_DEBUG_ANY,
 		    "Error: do_unbind: ldap_unbind failed for %s:%d: %s\n",
@@ -748,19 +768,20 @@ do_bind(
 
     if ( ri == NULL ) {
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: null ri ptr\n" ));
+	LDAP_LOG ( OPERATION, ERR, "do_bind: null ri ptr\n" , 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY, "Error: do_bind: null ri ptr\n", 0, 0, 0 );
 #endif
 	return( BIND_ERR_BADRI );
     }
 
+retry:
     if ( ri->ri_ldp != NULL ) {
 	ldrc = ldap_unbind( ri->ri_ldp );
 	if ( ldrc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
-			"do_bind: ldap_unbind failed: %s\n", ldap_err2string( ldrc ) ));
+		LDAP_LOG ( OPERATION, ERR, 
+			"do_bind: ldap_unbind failed: %s\n", ldap_err2string( ldrc ), 0, 0 );
 #else
 	    Debug( LDAP_DEBUG_ANY,
 		    "Error: do_bind: ldap_unbind failed: %s\n",
@@ -769,11 +790,35 @@ do_bind(
 	}
 	ri->ri_ldp = NULL;
     }
-
+    
+	if ( ri->ri_uri != NULL ) { /* new URI style */
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ARGS, 
+		LDAP_LOG ( OPERATION, ARGS, 
+			"do_bind: Initializing session to %s\n", 
+		    ri->ri_uri, 0, 0);
+#else
+	    Debug( LDAP_DEBUG_ARGS, "Initializing session to %s\n",
+		    ri->ri_uri, 0, 0 );
+#endif
+
+		ldrc = ldap_initialize( &(ri->ri_ldp), ri->ri_uri);
+
+		if (ldrc != LDAP_SUCCESS) {
+#ifdef NEW_LOGGING
+		LDAP_LOG ( OPERATION, ERR, 
+			"do_bind: ldap_initalize (0, %s) failed: %s\n",
+			ri->ri_uri, ldap_err2string(ldrc), 0 );
+#else
+		Debug( LDAP_DEBUG_ANY, "Error: ldap_initialize(0, %s) failed: %s\n",
+			ri->ri_uri, ldap_err2string(ldrc), 0 );
+#endif
+		return( BIND_ERR_OPEN );		
+		}
+	} else { /* old HOST style */
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ARGS, 
 		"do_bind: Initializing session to %s:%d\n", 
-	    ri->ri_hostname, ri->ri_port ));
+	    ri->ri_hostname, ri->ri_port, 0 );
 #else
     Debug( LDAP_DEBUG_ARGS, "Initializing session to %s:%d\n",
 	    ri->ri_hostname, ri->ri_port, 0 );
@@ -782,14 +827,15 @@ do_bind(
     ri->ri_ldp = ldap_init( ri->ri_hostname, ri->ri_port );
     if ( ri->ri_ldp == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, 
+		LDAP_LOG ( OPERATION, ERR, 
 			"do_bind: ldap_init (%s, %d) failed: %s\n",
-			ri->ri_hostname, ri->ri_port, sys_errlist[ errno ] ));
+			ri->ri_hostname, ri->ri_port, sys_errlist[ errno ] );
 #else
 		Debug( LDAP_DEBUG_ANY, "Error: ldap_init(%s, %d) failed: %s\n",
 			ri->ri_hostname, ri->ri_port, sys_errlist[ errno ] );
 #endif
 		return( BIND_ERR_OPEN );
+    }
     }
 
 	{	/* set version 3 */
@@ -799,9 +845,9 @@ do_bind(
 
 		if( err != LDAP_OPT_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: ",
+			LDAP_LOG ( OPERATION, ERR, "do_bind: "
 				"Error: ldap_set_option(%s, LDAP_OPT_VERSION, 3) failed!\n",
-				ri->ri_hostname ));
+				ri->ri_hostname, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"Error: ldap_set_option(%s, LDAP_OPT_VERSION, 3) failed!\n",
@@ -824,9 +870,9 @@ do_bind(
 
 		if( err != LDAP_OPT_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: ",
+			LDAP_LOG ( OPERATION, ERR, "do_bind: "
 				"Error: ldap_set_option(%s, REFERRALS, OFF) failed!\n",
-				ri->ri_hostname ));
+				ri->ri_hostname, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"Error: ldap_set_option(%s,REFERRALS, OFF) failed!\n",
@@ -840,15 +886,14 @@ do_bind(
 	ldap_set_option(ri->ri_ldp, LDAP_OPT_RESTART, LDAP_OPT_ON);
 
 	if( ri->ri_tls ) {
-		int err;
-		err = ldap_start_tls_s(ri->ri_ldp, NULL, NULL);
+		int err = ldap_start_tls_s(ri->ri_ldp, NULL, NULL);
 
 		if( err != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: ",
+			LDAP_LOG ( OPERATION, ERR, "do_bind: "
 				"%s: ldap_start_tls failed: %s (%d)\n",
 				ri->ri_tls == TLS_CRITICAL ? "Error" : "Warning",
-				ldap_err2string( err ), err ));
+				ldap_err2string( err ), err );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"%s: ldap_start_tls failed: %s (%d)\n",
@@ -857,10 +902,13 @@ do_bind(
 #endif
 
 			if( ri->ri_tls == TLS_CRITICAL ) {
+				*lderr = err;
 				ldap_unbind( ri->ri_ldp );
 				ri->ri_ldp = NULL;
 				return BIND_ERR_TLS_FAILED;
 			}
+			ri->ri_tls = TLS_OFF;
+			goto retry;
 		}
 	}
 
@@ -870,9 +918,9 @@ do_bind(
 	 * Bind with a plaintext password.
 	 */
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ARGS, 
+	LDAP_LOG ( OPERATION, ARGS, 
 		"do_bind: bind to %s:%d as %s (simple)\n", 
-		ri->ri_hostname, ri->ri_port, ri->ri_bind_dn ));
+		ri->ri_hostname, ri->ri_port, ri->ri_bind_dn );
 #else
 	Debug( LDAP_DEBUG_ARGS, "bind to %s:%d as %s (simple)\n",
 		ri->ri_hostname, ri->ri_port, ri->ri_bind_dn );
@@ -881,9 +929,9 @@ do_bind(
 		ri->ri_password );
 	if ( ldrc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: "
+		LDAP_LOG ( OPERATION, ERR, "do_bind: "
 		    "Error: ldap_simple_bind_s for %s:%d failed: %s\n",
-		    ri->ri_hostname, ri->ri_port, ldap_err2string( ldrc ) ));
+		    ri->ri_hostname, ri->ri_port, ldap_err2string( ldrc ) );
 #else
 	    Debug( LDAP_DEBUG_ANY,
 		    "Error: ldap_simple_bind_s for %s:%d failed: %s\n",
@@ -898,25 +946,28 @@ do_bind(
 
 	case AUTH_SASL:
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ARGS, 
+	LDAP_LOG ( OPERATION, ARGS, 
 		"do_bind: bind to %s as %s via %s (SASL)\n", 
-		ri->ri_hostname, ri->ri_authcId, ri->ri_saslmech ));
+		ri->ri_hostname,
+		ri->ri_authcId ? ri->ri_authcId : "-",
+		ri->ri_saslmech );
 #else
 	Debug( LDAP_DEBUG_ARGS, "bind to %s as %s via %s (SASL)\n",
-		ri->ri_hostname, ri->ri_authcId, ri->ri_saslmech );
+		ri->ri_hostname,
+		ri->ri_authcId ? ri->ri_authcId : "-",
+		ri->ri_saslmech );
 #endif
 
 #ifdef HAVE_CYRUS_SASL
 	if( ri->ri_secprops != NULL ) {
-		int err;
-		err = ldap_set_option(ri->ri_ldp, LDAP_OPT_X_SASL_SECPROPS,
-			ri->ri_secprops);
+		int err = ldap_set_option(ri->ri_ldp,
+			LDAP_OPT_X_SASL_SECPROPS, ri->ri_secprops);
 
 		if( err != LDAP_OPT_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: "
+			LDAP_LOG ( OPERATION, ERR, "do_bind: "
 				"Error: ldap_set_option(%s,SECPROPS,\"%s\") failed!\n",
-				ri->ri_hostname, ri->ri_secprops ));
+				ri->ri_hostname, ri->ri_secprops, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"Error: ldap_set_option(%s,SECPROPS,\"%s\") failed!\n",
@@ -929,18 +980,19 @@ do_bind(
 	}
 
 	{
-		char *passwd = ri->ri_password ? ber_strdup( ri->ri_password ) : NULL;
 		void *defaults = lutil_sasl_defaults( ri->ri_ldp, ri->ri_saslmech,
-		    ri->ri_realm, ri->ri_authcId, passwd, ri->ri_authzId );
+		    ri->ri_realm, ri->ri_authcId, ri->ri_password, ri->ri_authzId );
 
 		ldrc = ldap_sasl_interactive_bind_s( ri->ri_ldp, ri->ri_bind_dn,
 		    ri->ri_saslmech, NULL, NULL,
 		    LDAP_SASL_QUIET, lutil_sasl_interact, defaults );
+
+		lutil_sasl_freedefs( defaults );
 		if ( ldrc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: "
+			LDAP_LOG ( OPERATION, ERR, "do_bind: "
 				"Error: LDAP SASL for %s:%d failed: %s\n",
-			    ri->ri_hostname, ri->ri_port, ldap_err2string( ldrc ) ));
+			    ri->ri_hostname, ri->ri_port, ldap_err2string( ldrc ) );
 #else
 			Debug( LDAP_DEBUG_ANY, "Error: LDAP SASL for %s:%d failed: %s\n",
 			    ri->ri_hostname, ri->ri_port, ldap_err2string( ldrc ));
@@ -950,16 +1002,13 @@ do_bind(
 			ri->ri_ldp = NULL;
 			return( BIND_ERR_SASL_FAILED );
 		}
-
-		ber_memfree( passwd );
-		ber_memfree( defaults );
 	}
 	break;
 #else
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: "
+	LDAP_LOG ( OPERATION, ERR, "do_bind: "
 		"Error: do_bind: SASL not supported %s:%d\n",
-		 ri->ri_hostname, ri->ri_port ));
+		ri->ri_hostname, ri->ri_port, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY,
 		"Error: do_bind: SASL not supported %s:%d\n",
@@ -972,9 +1021,9 @@ do_bind(
 
     default:
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: "
+	LDAP_LOG ( OPERATION, ERR, "do_bind: "
 		"Error: do_bind: unknown auth type \"%d\" for %s:%d\n",
-		ri->ri_bind_method, ri->ri_hostname, ri->ri_port ));
+		ri->ri_bind_method, ri->ri_hostname, ri->ri_port );
 #else
 	Debug(  LDAP_DEBUG_ANY,
 		"Error: do_bind: unknown auth type \"%d\" for %s:%d\n",
@@ -1001,9 +1050,9 @@ do_bind(
 
 		if( err != LDAP_OPT_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "ldap_op", LDAP_LEVEL_ERR, "do_bind: "
+			LDAP_LOG ( OPERATION, ERR, "do_bind: "
 				"ldap_set_option(%s, SERVER_CONTROLS, ManageDSAit) failed!\n",
-				ri->ri_hostname ));
+				ri->ri_hostname, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY, "Error: "
 				"ldap_set_option(%s, SERVER_CONTROLS, ManageDSAit) failed!\n",
@@ -1038,15 +1087,15 @@ dump_ldm_array(
     for ( i = 0; ldmarr[ i ] != NULL; i++ ) {
 	ldm = ldmarr[ i ];
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_INFO, "dump_ldm_array: "
+	LDAP_LOG ( OPERATION, INFO, "dump_ldm_array: "
 		"Trace (%ld): *** ldmarr[ %d ] contents:\n",
-		(long) getpid(), i ));
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_INFO, "dump_ldm_array: "
+		(long) getpid(), i, 0 );
+	LDAP_LOG ( OPERATION, INFO, "dump_ldm_array: "
 		"Trace (%ld): *** ldm->mod_op: %d\n",
-		(long) getpid(), ldm->mod_op ));
-	LDAP_LOG (( "ldap_op", LDAP_LEVEL_INFO, "dump_ldm_array: "
+		(long) getpid(), ldm->mod_op, 0 );
+	LDAP_LOG ( OPERATION, INFO, "dump_ldm_array: "
 		"Trace (%ld): *** ldm->mod_type: %s\n",
-		(long) getpid(), ldm->mod_type ));
+		(long) getpid(), ldm->mod_type, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"Trace (%ld): *** ldmarr[ %d ] contents:\n",
@@ -1064,8 +1113,8 @@ dump_ldm_array(
 		sprintf( msgbuf, "***** bv[ %d ] len = %ld, val = <%s>",
 			j, b->bv_len, b->bv_val );
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ldap_op", LDAP_LEVEL_INFO, "dump_ldm_array: "
-			"Trace (%ld):%s\n", (long) getpid(), msgbuf ));
+		LDAP_LOG ( OPERATION, INFO, "dump_ldm_array: "
+			"Trace (%ld):%s\n", (long) getpid(), msgbuf, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"Trace (%ld):%s\n", (long) getpid(), msgbuf, 0 );
