@@ -1,23 +1,13 @@
-# $OpenLDAP: pkg/ldap/tests/scripts/args.sh,v 1.1.2.1 2003/09/18 15:43:41 kurt Exp $
+# $OpenLDAP: pkg/ldap/tests/scripts/args.sh,v 1.1.2.3 2003/10/13 01:57:04 kurt Exp $
 
 BACKEND=bdb
 if test $# -ge 1 ; then
         BACKEND=$1; shift
 fi
 
-BACKENDTYPE=yes
+SYNCREPL=no
 if test $# -ge 1 ; then
-        BACKENDTYPE=$1; shift
-fi
-
-MONITORDB=no
-if test $# -ge 1 ; then
-        MONITORDB=$1; shift
-fi
-
-PROXYCACHE=no
-if test $# -ge 1 ; then
-        PROXYCACHE=$1; shift
+	SYNCREPL=$1; shift
 fi
 
 WAIT=0
